/*
 * ColorMapGenerator.java
 *
 * ex command line: 50 0 150 or 50
 */

package sddm;

import java.io.*;

/**
 *
 * @author Erik J. Tollerud
 */
public class ColorMapGenerator {
    public int numberOfShades;
    public float startRange;
    public float endRange;
    
    public ColorMapGenerator() {
        numberOfShades = 2;
        startRange = 0;
        endRange = 0;
    }
    
    public boolean getDirect() {
        return (startRange != endRange);
    }
    
    public void setToIndirect() {
        startRange = 0;
        endRange = 0;
    }
    
    public void setNumberOfShades(int number) {
        this.numberOfShades = number;
    }
    
    public int getNumberOfShades() {
        return this.numberOfShades;
    }
    
    public void setStartOfRange(float start) {
        this.startRange = start;
    }
    
    public float getStartOfRange() {
        return this.startRange;
    }
    
    public void setEndOfRange(float end) {
        this.endRange = end;
    }
    
    public float getEndOfRange() {
        return this.endRange;
    }
    
    public String makeText() {
        StringBuffer sb;
        
        if (startRange == endRange) {
            sb = new StringBuffer("0 0 0 1 ");
            for(int i = 1; i <= numberOfShades;++i) {
                String greyNum = Float.toString((float)i/(float)numberOfShades);
                sb.append(greyNum);
                sb.append(' ');
                sb.append(greyNum);
                sb.append(' ');
                sb.append(greyNum);
                sb.append(' ');
                sb.append(1);
                sb.append(' ');
            }
        }
        
        else {
            sb = new StringBuffer(startRange + " 0 0 0 1 ");
            float range = endRange - startRange;
           for(int i = 1; i <= numberOfShades;++i) {
                float scaledVal = (float)i/(float)numberOfShades;
                String greyNum = Float.toString(scaledVal);
                sb.append(scaledVal*range+startRange);
                sb.append(' ');
                sb.append(greyNum);
                sb.append(' ');
                sb.append(greyNum);
                sb.append(' ');
                sb.append(greyNum);
                sb.append(' ');
                sb.append(1);
                sb.append(' ');
            } 
        }
        return sb.toString();
    }
    
    //arguments: numberOfShades | numberOfShades filename | numberOfShades startRange endRange | numberOfShades startRange endRange filename
    public static void main(String[] args) {
        File fileToWrite = null;
        ColorMapGenerator cmg = new ColorMapGenerator();
        
        if (args.length==1)
            cmg.setNumberOfShades(Integer.parseInt(args[0]));
        else if (args.length == 2) {
            cmg.setNumberOfShades(Integer.parseInt(args[0]));
            fileToWrite = new File(args[1]);
        }
        else if (args.length == 3) {
            cmg.setNumberOfShades(Integer.parseInt(args[0]));
            cmg.setStartOfRange(Float.parseFloat(args[1]));
            cmg.setEndOfRange(Float.parseFloat(args[2]));
        }
        else if (args.length == 4) {
            cmg.setNumberOfShades(Integer.parseInt(args[0]));
            cmg.setStartOfRange(Float.parseFloat(args[1]));
            cmg.setEndOfRange(Float.parseFloat(args[2]));
            fileToWrite = new File(args[1]);
        }
        
        if (fileToWrite == null) {
            javax.swing.JFileChooser jfc = new javax.swing.JFileChooser();
            jfc.setMultiSelectionEnabled(false);
            if(jfc.showSaveDialog(null)==jfc.APPROVE_OPTION)
                fileToWrite = jfc.getSelectedFile();
        }
        
        if (fileToWrite != null) {
            if(fileToWrite.exists()) {
                if (javax.swing.JOptionPane.showConfirmDialog(null,"File exists - overwrite?","File exists",javax.swing.JOptionPane.YES_NO_OPTION) == javax.swing.JOptionPane.YES_OPTION)
                    fileToWrite.delete();
            }
            if(!fileToWrite.exists()) {
                try{
                    fileToWrite.createNewFile();
                    FileWriter fw = new FileWriter(fileToWrite);
                    System.out.println("Writing to file "+fileToWrite.toString());
                    fw.write(cmg.makeText());
                    fw.flush();
                    fw.close();
                } catch (Exception e) {
                    javax.swing.JOptionPane.showMessageDialog(null,"Exception thrown writing to file:"+e.getMessage());
                }
            }
        }
        
        System.out.println("Exiting...");
    }
    
}
