/*
 * SDDMListener.java
 *
 * Created on June 12, 2006, 1:01 AM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package sddm;
import javax.media.opengl.*;
import java.nio.*;
import java.io.*;
import sddm.SDDModule;

/**
 *
 * @author Erik J. Tollerud
 */
public class SDDMListener implements GLEventListener{
    enum StereoMethod{OPENGL,DUAL_WINDOW_LEFT,DUAL_WINDOW_RIGHT,ANAGLYPH,ANAGLYPH_FORCE_BASIC,NONE};
    private enum AnaglyphMethod{MEMORY, MEMORY_ACCUM, TEXTURE_MODULATE, ARB, BUILTIN, COLOR_MATRIX_AUXBUFF, COLOR_MATRIX_2BUFF, COLOR_MATRIX_PBUFF};
    
    
    private StereoCamera camera;
    private StereoMethod method;
    private SDDModule module;
    private File ssFile;
    private int mainDisplayList;
    private int luminizerShaderProgram;
    float[] colorMatrix;
    static int numOfInstances = 0;
    private static int numOfListenersToRecompile = 0;
    
    
    private boolean debugEnabled;
    private AnaglyphMethod anaglyphMethod = AnaglyphMethod.MEMORY;
    private boolean measureFps = true;
    //Anaglyph colors in float form for rapid access
    private float lColorRed, lColorGreen, lColorBlue, rColorRed, rColorBlue, rColorGreen;
    
    //Pbuffer for COLOR_MATRIX anaglyph mode
    GLPbuffer anaPBuffer;
    
    /** Creates a new instance of SDDMListener */
    public SDDMListener(SDDModule mod, StereoMethod sm) {
        this(mod,sm,new StereoCamera());
    }
    public SDDMListener(SDDModule mod, StereoMethod sm, StereoCamera camera) {
        this.camera = camera;
        this.method = sm;
        this.module = mod;
        debugEnabled = false;
        //Default is red/blue for right/left
        rColorRed = 1;
        rColorGreen = rColorBlue = 0;
        lColorBlue = 1;
        rColorRed = rColorGreen = 0;
        ssFile = null;
        ++numOfListenersToRecompile;
        ++numOfInstances;
    }
    
    public void reshape(GLAutoDrawable drawable, int x, int y, int width, int height) {
        //TODO:determine if there is a need to force all attached frames to match shapes
        StereoCamera.FrustumSettings fs = this.camera.getFrustumSettings();
        fs.screenAspectRatio = (float)width/(float)height;
        this.camera.setFrustumSettings(fs);
        this.module.resize(drawable.getGL(),x,y,width,height);
    }
    
    public void init(GLAutoDrawable drawable) {
        this.module.init(drawable.getGL());
        if (debugEnabled)
            drawable.setGL(new javax.media.opengl.DebugGL(drawable.getGL()));
        
        GL gl = drawable.getGL();
        mainDisplayList = gl.glGenLists(1);
        
        //Necessary for when GLContext is removed and re-built to make sure the list re-compiles
        module.recompileDisplayList();
        
        //Set up anaglyph vertex shader program
        if (method == StereoMethod.ANAGLYPH) {
            
            if (!setupShaderAna(drawable)) {
                if(!setupColorMatrixAna(drawable)) {
                    this.anaglyphMethod = AnaglyphMethod.MEMORY;
                    System.err.println("Couldn't use fast anaglyph techniques - defaulting to SLOW main memory method.");
                }
            }
            
            //Uncomment one of these for testing without the shader
            //this.anaglyphMethod = AnaglyphMethod.COLOR_MATRIX_AUXBUFF;this.colorMatrix = new float[16];
            //this.anaglyphMethod = AnaglyphMethod.COLOR_MATRIX_2BUFF;this.colorMatrix = new float[16];drawable.setAutoSwapBufferMode(false);
            //this.anaglyphMethod = AnaglyphMethod.MEMORY;
        } else if (method== StereoMethod.ANAGLYPH_FORCE_BASIC) {
            this.anaglyphMethod = AnaglyphMethod.MEMORY;
            System.out.println("Using basic Memory anaglyph technique by user request.");
        }
        
    }
    
    private boolean setupShaderAna(GLAutoDrawable drawable) {
        GL gl = drawable.getGL();
        
        try {
            if(gl.isFunctionAvailable("glCreateShader")) {
                this.anaglyphMethod = AnaglyphMethod.BUILTIN;
                System.out.println("Using OpenGL 2.0 Builtin GLSL for Anaglyph Rendering");
                luminizerShaderProgram = configLuminizer(gl,false);
                return true;
            } else if (gl.isFunctionAvailable("glCreateShaderObjectARB")) {
                this.anaglyphMethod = AnaglyphMethod.ARB;
                System.out.println("Using GLSL Extensions for OpenGL for Anaglyph Rendering");
                luminizerShaderProgram = configLuminizer(gl,true);
                return true;
            }
        } catch(GLException e) {
            System.err.println("Error setting up shader, using alternative anaglyph methods");
        }
        return false;
    }
    
    private boolean setupColorMatrixAna(GLAutoDrawable drawable) {
        return setupColorMatrixAna(drawable, false);
    }
    
    private boolean setupColorMatrixAna(GLAutoDrawable drawable, boolean requireAux) {
        GL gl = drawable.getGL();
        
        if (!(gl.isExtensionAvailable("GL_SGI_color_matrix")||gl.isExtensionAvailable("GL_ARB_imaging")))
            return false;
        else {
            int[] auxBuffNum = new int[1];
            gl.glGetIntegerv(GL.GL_AUX_BUFFERS,auxBuffNum,0);
            if(auxBuffNum[0] > 0) {
                this.anaglyphMethod = AnaglyphMethod.COLOR_MATRIX_AUXBUFF;
                System.out.println("Using Color Matrix With Auxiliary Buffer");
            } else if (requireAux) {
                System.err.println("Color Matrix was supported, but no Aux buffer Available");
                return false;
            } else  if (false&&GLDrawableFactory.getFactory().canCreateGLPbuffer()) {
                this.anaglyphMethod = AnaglyphMethod.COLOR_MATRIX_PBUFF;
                //anaPBuffer = GLDrawableFactory.getFactory().createGLPbuffer();
                //TODO: implement PBuffer technique and remove false marker above
                System.out.println("Using Color Matrix With Pbuffer");
            } else if (true) {
                //TODO: figure out how to do test for double-bufferedness, or just assume that they're all 2B
                this.anaglyphMethod = AnaglyphMethod.COLOR_MATRIX_2BUFF;
                drawable.setAutoSwapBufferMode(false);
                System.out.println("Using Color Matrix With Front/Back Buffer");
            } else {
                System.err.println("Color Matrix was supported, but no extra buffers Available");
                return false;
            }
        }
        
        this.colorMatrix = new float[16];
        this.colorMatrix[0] = 1.0f;
        this.colorMatrix[5] = 1.0f;
        this.colorMatrix[10] = 1.0f;
        this.colorMatrix[15] = 1.0f;
        
        
        
        return true;
    }
    
    private int configLuminizer(GL gl, boolean arb) {
        //program source
        final String[] vertSource={
            "void main()\n",
            "{\n",
            "gl_FrontColor = gl_Color;\n",
            "gl_Position = ftransform();\n",
            "gl_TexCoord[0] = gl_MultiTexCoord0;\n",
            "}\n"
        };
        final String[] fragSource={
            "uniform sampler2D screenCap;\n",
            "void main()\n",
            "{\n",
            "vec2 texCoord= gl_TexCoord[0].st;\n",
            "vec4 texColor = texture2D(screenCap,texCoord);\n",
            "float lum = (texColor.r+texColor.g+texColor.b)/3.0;\n",
            //changed due to eye luminance perception?
            //"float lum = 0.299*texColor.r+0.587*texColor.g+0.114*texColor.b;\n",
            "vec3 mainColor = lum*gl_Color.rgb;\n",
            "gl_FragColor = vec4(mainColor,texColor.a);\n",
            "}\n"
        };
        
        int prog = -1;
        if (arb) {
            int fragShader = gl.glCreateShaderObjectARB(gl.GL_FRAGMENT_SHADER);
            int vertShader = gl.glCreateShaderObjectARB(gl.GL_VERTEX_SHADER);
            
            int[] fragLengths = new int[fragSource.length];
            for (int i=0; i < fragLengths.length;++i)
                fragLengths[i] = fragSource[i].length();
            int[] vertLengths = new int[vertSource.length];
            for (int i=0; i < vertLengths.length;++i)
                vertLengths[i] = vertSource[i].length();
            
            gl.glShaderSourceARB(fragShader,fragSource.length,fragSource,fragLengths,0);
            gl.glShaderSourceARB(vertShader,vertSource.length,vertSource,vertLengths,0);
            
            gl.glCompileShaderARB(vertShader);
            gl.glCompileShaderARB(fragShader);
            
            prog = gl.glCreateProgramObjectARB();
            
            gl.glAttachObjectARB(prog,vertShader);
            gl.glAttachObjectARB(prog,fragShader);
            
            gl.glLinkProgramARB(prog);
        } else {
            int fragShader = gl.glCreateShader(gl.GL_FRAGMENT_SHADER);
            int vertShader = gl.glCreateShader(gl.GL_VERTEX_SHADER);
            
            int[] fragLengths = new int[fragSource.length];
            for (int i=0; i < fragLengths.length;++i)
                fragLengths[i] = fragSource[i].length();
            int[] vertLengths = new int[vertSource.length];
            for (int i=0; i < vertLengths.length;++i)
                vertLengths[i] = vertSource[i].length();
            
            gl.glShaderSource(fragShader,fragSource.length,fragSource,fragLengths,0);
            gl.glShaderSource(vertShader,vertSource.length,vertSource,vertLengths,0);
            
            gl.glCompileShader(vertShader);
            gl.glCompileShader(fragShader);
            
            prog = gl.glCreateProgram();
            
            gl.glAttachShader(prog,vertShader);
            gl.glAttachShader(prog,fragShader);
            
            gl.glLinkProgram(prog);
        }
        return prog;
    }
    
    public void display(GLAutoDrawable drawable) {
        long startTime = 0;
        long endTime = 0;
        if(measureFps)
            startTime = System.nanoTime();
        GL gl = drawable.getGL();
        
        module.applyClearColor(gl);
        boolean dl = module.isDisplayListOn();
        if (dl&&module.recompileDisplayList) {
            gl.glNewList(mainDisplayList,GL.GL_COMPILE);
            module.drawGeometry(gl);
            gl.glEndList();
            if (--numOfListenersToRecompile <= 0) {
                module.recompileDisplayList = false;
                numOfListenersToRecompile = numOfInstances;
            }
            // module.recompileDisplayList = false;
        }
        int[] viewportInfo = new int[4];
        gl.glGetIntegerv(gl.GL_VIEWPORT, viewportInfo,0);
        int frameWidth = viewportInfo[2];
        int frameHeight = viewportInfo[3];
        int dataSize = frameWidth * frameHeight;
        int frameX = viewportInfo[0];
        int frameY = viewportInfo[1];
        
        gl.glClear(gl.GL_DEPTH_BUFFER_BIT|gl.GL_COLOR_BUFFER_BIT);
        switch(this.method) {
            case OPENGL:
                //Draw twice for each eye, drawing to the correct buffer
                for (int i = 0; i <2; ++i) {
                    if (this.camera.isRight())
                        gl.glDrawBuffer(gl.GL_RIGHT);
                    else
                        gl.glDrawBuffer(gl.GL_LEFT);
                    this.camera.updateCamera(gl);
                    module.preDraw(gl);
                    if (dl)
                        gl.glCallList(mainDisplayList);
                    else
                        this.module.drawGeometry(gl);
                    this.module.postDraw(gl);
                }
                break;
            case ANAGLYPH:
            case ANAGLYPH_FORCE_BASIC:
                switch (this.anaglyphMethod) {
                    
                    case BUILTIN:
                    case ARB:
                        //Vertex Shader method
                        gl.glDrawBuffer(gl.GL_BACK);
                        gl.glReadBuffer(gl.GL_BACK);
                        
                        gl.glClear(gl.GL_ACCUM_BUFFER_BIT);
                        
                        for (int i = 0; i < 2; ++i) {
                            gl.glClear(gl.GL_COLOR_BUFFER_BIT|gl.GL_DEPTH_BUFFER_BIT);
                            boolean onRight = this.camera.isRight();
                            
                            //draw module
                            this.camera.updateCamera(gl);
                            module.preDraw(gl);
                            if (dl)
                                gl.glCallList(mainDisplayList);
                            else
                                this.module.drawGeometry(gl);
                            this.module.postDraw(gl);
                            
                            
                            
                            //push/pop settings in case module requires it
                            gl.glPushAttrib(gl.GL_ENABLE_BIT|gl.GL_TEXTURE_BIT|gl.GL_POLYGON_BIT);
                            
                            gl.glBindTexture(GL.GL_TEXTURE_2D,0);
                            gl.glTexParameterf(gl.GL_TEXTURE_2D,gl.GL_TEXTURE_MAG_FILTER,gl.GL_NEAREST);
                            gl.glTexParameterf(gl.GL_TEXTURE_2D,gl.GL_TEXTURE_MIN_FILTER,gl.GL_NEAREST);
                            
                            //setup texture environment and enable
                            gl.glEnable(gl.GL_TEXTURE_2D);
                            gl.glTexEnvf(gl.GL_TEXTURE_ENV, gl.GL_TEXTURE_ENV_MODE,gl.GL_DECAL);
                            
                            float xTexCoordEdge = 1.0f;
                            float yTexCoordEdge = 1.0f;
                            //Grab whole scene into current texture. Also set texture coordinate edges.
                            if (this.anaglyphMethod == AnaglyphMethod.ARB) {
                                //must use power of two texture sizes if not OpenGL 2.0
                                int width = viewportInfo[2];
                                int height = viewportInfo[3];
                                
                                int j = 2;
                                while(j < width)
                                    j*=2;
                                xTexCoordEdge = (float)width/(float)j;
                                width = j;
                                j = 2;
                                while(j < height)
                                    j*=2;
                                yTexCoordEdge = (float)height/(float)j;
                                height = j;
                                
                                gl.glCopyTexImage2D(gl.GL_TEXTURE_2D, 0, gl.GL_RGB, viewportInfo[0], viewportInfo[1], width, height, 0);
                            } else
                                gl.glCopyTexImage2D(gl.GL_TEXTURE_2D, 0, gl.GL_RGB, viewportInfo[0], viewportInfo[1], viewportInfo[2], viewportInfo[3], 0);
                            
                            //Necessary because some modules may change these settings
                            gl.glPolygonMode(gl.GL_FRONT_AND_BACK,gl.GL_FILL);
                            gl.glDisable(gl.GL_DEPTH_TEST);
                            gl.glDisable(gl.GL_CULL_FACE);
                            
                            if (this.anaglyphMethod == AnaglyphMethod.ARB)
                                gl.glUseProgramObjectARB(luminizerShaderProgram);
                            else
                                gl.glUseProgram(luminizerShaderProgram);
                            
                            //Draw single quad filling screen with the texture mapped onto it, modulating the eye color
                            if (onRight)
                                gl.glColor3f(this.rColorRed,this.rColorGreen,this.rColorBlue);
                            else
                                gl.glColor3f(this.lColorRed,this.lColorGreen,this.lColorBlue);
                            
                            gl.glMatrixMode(gl.GL_PROJECTION);
                            gl.glLoadIdentity();
                            gl.glMatrixMode(gl.GL_MODELVIEW);
                            gl.glLoadIdentity();
                            
                            gl.glBegin(gl.GL_QUADS);
                            //lower-left
                            gl.glTexCoord2f(0,0);
                            gl.glVertex2f(-1,-1);
                            //lower-right
                            gl.glTexCoord2f(xTexCoordEdge,0);
                            gl.glVertex2f(1,-1);
                            //upper-right
                            gl.glTexCoord2f(xTexCoordEdge,yTexCoordEdge);
                            gl.glVertex2f(1,1);
                            //upper-left
                            gl.glTexCoord2f(0,yTexCoordEdge);
                            gl.glVertex2f(-1,1);
                            gl.glEnd();
                            
                            if (this.anaglyphMethod == AnaglyphMethod.ARB)
                                gl.glUseProgramObjectARB(0);
                            else
                                gl.glUseProgram(0);
                            
                            gl.glPopAttrib();
                            
                            //send to accumulation buffer
                            gl.glAccum(gl.GL_ACCUM,1.0f);
                        }
                        
                        //write accumulated result to frame buffer
                        //TODOgl.glClear(gl.GL_COLOR_BUFFER_BIT | gl.GL_DEPTH_BUFFER_BIT);
                        gl.glAccum(gl.GL_RETURN,1.0f);
                        break;
                    case COLOR_MATRIX_PBUFF:
                    case COLOR_MATRIX_2BUFF:
                    case COLOR_MATRIX_AUXBUFF:
                        int drawBuffer=GL.GL_BACK;
                        int renderBuffer=GL.GL_FRONT;
                        switch(this.anaglyphMethod) {
                            case COLOR_MATRIX_PBUFF:
                                //TODO: implement PBuffer technique
                                break;
                            case COLOR_MATRIX_2BUFF:
                                drawBuffer = GL.GL_FRONT;
                                renderBuffer = GL.GL_BACK;
                                break;
                            case COLOR_MATRIX_AUXBUFF:
                                drawBuffer = GL.GL_BACK;
                                renderBuffer = GL.GL_AUX0;
                                break;
                        }
                        
                        //TODO:test, and find a way to not do the matrix every cycle if it works
                        gl.glDrawBuffer(drawBuffer);
                            gl.glClear(gl.GL_DEPTH_BUFFER_BIT|gl.GL_COLOR_BUFFER_BIT);
                        // twice to get both eyes
                        for (int i=0;i<2;i++) {
                            
                            gl.glDrawBuffer(renderBuffer);
                          
                            this.camera.updateCamera(gl);
                            module.preDraw(gl);
                            
                            if (dl)
                                gl.glCallList(mainDisplayList);
                            else
                                this.module.drawGeometry(gl);
                            this.module.postDraw(gl);
                            
                            //Set matrix to load so that the colors all map to the correct anaglyph color
                            if (this.camera.isRight()) {
                                colorMatrix[0] = this.rColorRed;
                                colorMatrix[4] = this.rColorRed;
                                colorMatrix[8] = this.rColorRed;
                                colorMatrix[1] = this.rColorGreen;
                                colorMatrix[5] = this.rColorGreen;
                                colorMatrix[9] = this.rColorGreen;
                                colorMatrix[2] = this.rColorBlue;
                                colorMatrix[6] = this.rColorBlue;
                                colorMatrix[10] = this.rColorBlue;
                            } else {
                                colorMatrix[0] = this.lColorRed;
                                colorMatrix[4] = this.lColorRed;
                                colorMatrix[8] = this.lColorRed;
                                colorMatrix[1] = this.lColorGreen;
                                colorMatrix[5] = this.lColorGreen;
                                colorMatrix[9] = this.lColorGreen;
                                colorMatrix[2] = this.lColorBlue;
                                colorMatrix[6] = this.lColorBlue;
                                colorMatrix[10] = this.lColorBlue;
                            }
                            //Get half of the alpha for blending - assumes all rendering is done with alpha = 1
                            
                            
                            gl.glPushAttrib(GL.GL_TRANSFORM_BIT | GL.GL_COLOR_BUFFER_BIT | GL.GL_PIXEL_MODE_BIT);
                            gl.glPixelTransferf(GL.GL_ALPHA_SCALE,0.5f);
                            gl.glEnable(GL.GL_BLEND);
                            gl.glBlendFunc(GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA);
                            gl.glMatrixMode(GL.GL_COLOR);
                            gl.glLoadMatrixf(colorMatrix,0);
                            
                            windowPos2iWorkaround(gl,0,0);
                            
                            gl.glReadBuffer(renderBuffer);
                            gl.glDrawBuffer(drawBuffer);
                            gl.glCopyPixels(viewportInfo[0],viewportInfo[1],viewportInfo[2],viewportInfo[3],GL.GL_COLOR);
                            
                            gl.glPopAttrib();
                        }
                        break;
                        
                    case MEMORY:
                        //Generates each eye, copies from frame buffer to memory, calculates pixels from each, draws to buffer
                        //Setbackground to black to not screw up eyes
                        gl.glReadBuffer(gl.GL_BACK);
                        
                        ByteBuffer rightLumBuffer = ByteBuffer.allocate(dataSize);
                        ByteBuffer leftLumBuffer = ByteBuffer.allocate(dataSize);
                        
                        //do twice for two eyes
                        for (int i = 0; i < 2;++i) {
                            //select the array to use for this iteration
                            ByteBuffer currentLumBuffer;
                            if(this.camera.isRight())
                                currentLumBuffer = rightLumBuffer;
                            else
                                currentLumBuffer = leftLumBuffer;
                            
                            this.camera.updateCamera(gl);
                            
                            gl.glClearColor(0.0f,0.0f,0.0f,0.0f);
                            gl.glClear(GL.GL_COLOR_BUFFER_BIT | GL.GL_DEPTH_BUFFER_BIT);
                            module.preDraw(gl);
                            if (dl)
                                gl.glCallList(mainDisplayList);
                            else
                                this.module.drawGeometry(gl);
                            this.module.postDraw(gl);
                            //read luminance values from this buffer into LumArray
                            gl.glReadPixels(viewportInfo[0], viewportInfo[1], frameWidth, frameHeight, gl.GL_LUMINANCE, gl.GL_BYTE, currentLumBuffer);
                        }
                        //Colorize array for correct eye color
                        byte[] rightLumArray = rightLumBuffer.array();
                        byte[] leftLumArray = leftLumBuffer.array();
                        byte[] RGBArray = new byte[3*dataSize];
                        
                        for (int i = 0; i < dataSize; ++i) {
                            int j = 3*i;
                            RGBArray[j] = (byte)((rightLumArray[i]*rColorRed+leftLumArray[i]*lColorRed)/2);
                            RGBArray[j+1] = (byte)((rightLumArray[i]*rColorGreen+leftLumArray[i]*lColorGreen)/2);
                            RGBArray[j+2] = (byte)((rightLumArray[i]*rColorBlue+leftLumArray[i]*lColorBlue)/2);
                        }
                        
                        gl.glClear(gl.GL_COLOR_BUFFER_BIT | gl.GL_DEPTH_BUFFER_BIT);
                        windowPos2iWorkaround(gl,0,0);
                        
                        ByteBuffer RGBBuffer = ByteBuffer.wrap(RGBArray);
                        gl.glDrawPixels(frameWidth,frameHeight,gl.GL_RGB, gl.GL_BYTE,RGBBuffer);
                        break;
                    case MEMORY_ACCUM:
                        //memory draw/read method - WARNING: NOT CORRECT
                        ByteBuffer lumBuffer = ByteBuffer.allocateDirect(dataSize);
                        
                        gl.glClear(gl.GL_ACCUM_BUFFER_BIT);
                        gl.glDrawBuffer(gl.GL_BACK);
                        gl.glReadBuffer(gl.GL_BACK);
                        //once for each eye
                        
                        gl.glPushAttrib(gl.GL_PIXEL_MODE_BIT|gl.GL_CURRENT_BIT);
                        for (int i = 0; i < 2; ++i) {
                            gl.glClear(gl.GL_COLOR_BUFFER_BIT|gl.GL_DEPTH_BUFFER_BIT);
                            boolean onRight = this.camera.isRight();
                            
                            //render
                            this.camera.updateCamera(gl);
                            this.module.preDraw(gl);
                            if (dl)
                                gl.glCallList(mainDisplayList);
                            else
                                this.module.drawGeometry(gl);
                            this.module.postDraw(gl);
                            
                            
                            gl.glPixelTransferf(gl.GL_RED_SCALE,1.0f);
                            gl.glPixelTransferf(gl.GL_GREEN_SCALE,1.0f);
                            gl.glPixelTransferf(gl.GL_BLUE_SCALE,1.0f);
                            
                            gl.glReadPixels(frameX,frameY,frameWidth,frameHeight,gl.GL_LUMINANCE,gl.GL_BYTE,lumBuffer);
                            
                            //set color to transfer back
                            gl.glPixelTransferf(gl.GL_RED_SCALE,onRight ? rColorRed : lColorRed);
                            gl.glPixelTransferf(gl.GL_GREEN_SCALE,onRight ? rColorGreen : lColorGreen);
                            gl.glPixelTransferf(gl.GL_BLUE_SCALE,onRight ? rColorBlue : lColorBlue);
                            
                            gl.glClear(gl.GL_COLOR_BUFFER_BIT | gl.GL_DEPTH_BUFFER_BIT);
                            windowPos2iWorkaround(gl,0,0);
                            gl.glDrawPixels(frameWidth,frameHeight,gl.GL_LUMINANCE,gl.GL_BYTE,lumBuffer);
                            
                            //send to accumulation buffer
                            gl.glAccum(gl.GL_ACCUM,1.0f);
                        }
                        //write accumulated result to back buffer
                        gl.glClear(gl.GL_COLOR_BUFFER_BIT | gl.GL_DEPTH_BUFFER_BIT);
                        gl.glAccum(gl.GL_RETURN,1.0f);
                        gl.glPopAttrib();
                        break;
                    case TEXTURE_MODULATE:
                        //Texture-modulation method - WARNING: NOT CORRECT
                        gl.glDrawBuffer(gl.GL_BACK);
                        gl.glReadBuffer(gl.GL_BACK);
                        
                        gl.glClear(gl.GL_ACCUM_BUFFER_BIT);
                        
                        for (int i = 0; i < 2; ++i) {
                            gl.glClear(gl.GL_COLOR_BUFFER_BIT|gl.GL_DEPTH_BUFFER_BIT);
                            boolean onRight = this.camera.isRight();
                            
                            //draw module
                            this.camera.updateCamera(gl);
                            this.module.preDraw(gl);
                            if (dl)
                                gl.glCallList(mainDisplayList);
                            else
                                this.module.drawGeometry(gl);
                            this.module.postDraw(gl);
                            
                            //push/pop settings in case module requires it
                            gl.glPushAttrib(gl.GL_ENABLE_BIT|gl.GL_TEXTURE_BIT);
                            
                            gl.glTexParameterf(gl.GL_TEXTURE_2D,gl.GL_TEXTURE_MAG_FILTER,gl.GL_NEAREST);
                            gl.glTexParameterf(gl.GL_TEXTURE_2D,gl.GL_TEXTURE_MIN_FILTER,gl.GL_NEAREST);
                            
                            //setup texture environment and enable
                            gl.glEnable(gl.GL_TEXTURE_2D);
                            gl.glTexEnvf(gl.GL_TEXTURE_ENV, gl.GL_TEXTURE_ENV_MODE,gl.GL_MODULATE);
                            
                            //Grab whole scene into current texture in luminance mode.. instead do this with each color?
                            gl.glCopyTexImage2D(gl.GL_TEXTURE_2D, 0, gl.GL_LUMINANCE, viewportInfo[0], viewportInfo[1], viewportInfo[2], viewportInfo[3], 0);
                            
                            gl.glClear(gl.GL_COLOR_BUFFER_BIT|gl.GL_DEPTH_BUFFER_BIT);
                            
                            gl.glDisable(gl.GL_DEPTH_TEST);
                            gl.glDisable(gl.GL_CULL_FACE);
                            gl.glDisable(gl.GL_LIGHTING);
                            
                            //Draw single quad filling screen with the texture mapped onto it, modulating the eye color
                            if (onRight)
                                gl.glColor3f(this.rColorRed,this.rColorGreen,this.rColorBlue);
                            else
                                gl.glColor3f(this.lColorRed,this.lColorGreen,this.lColorBlue);
                            
                            gl.glMatrixMode(gl.GL_PROJECTION);
                            gl.glLoadIdentity();
                            gl.glMatrixMode(gl.GL_MODELVIEW);
                            gl.glLoadIdentity();
                            
                            gl.glBegin(gl.GL_QUADS);
                            //lower-left
                            gl.glTexCoord2f(0,0);
                            gl.glVertex2f(-1,-1);
                            //lower-right
                            gl.glTexCoord2f(1,0);
                            gl.glVertex2f(1,-1);
                            //upper-right
                            gl.glTexCoord2f(1,1);
                            gl.glVertex2f(1,1);
                            //upper-left
                            gl.glTexCoord2f(0,1);
                            gl.glVertex2f(-1,1);
                            gl.glEnd();
                            
                            gl.glPopAttrib();
                            
                            //send to accumulation buffer
                            gl.glAccum(gl.GL_ACCUM,1.0f);
                        }
                        
                        //write accumulated result to frame buffer
                        gl.glClear(gl.GL_COLOR_BUFFER_BIT | gl.GL_DEPTH_BUFFER_BIT);
                        gl.glAccum(gl.GL_RETURN,1.0f);
                        break;
                }
                break;
                
                
            case DUAL_WINDOW_LEFT:
            case DUAL_WINDOW_RIGHT:
                //Draw only once using the correct camera side
                this.camera.setRight(this.method==StereoMethod.DUAL_WINDOW_RIGHT);
                this.camera.updateCamera(gl);
                module.preDraw(gl);
                if (dl)
                    gl.glCallList(mainDisplayList);
                else
                    this.module.drawGeometry(gl);
                this.module.postDraw(gl);
                break;
            case NONE:
            default:
                this.camera.updateCamera(gl);
                module.preDraw(gl);
                
                if (dl)
                    gl.glCallList(mainDisplayList);
                else
                    this.module.drawGeometry(gl);
                this.module.postDraw(gl);
                
                /*gl.glMatrixMode(gl.GL_PROJECTION);
                gl.glLoadIdentity();
                gl.glMatrixMode(gl.GL_MODELVIEW);
                gl.glLoadIdentity();
                gl.glColor3f(1.0f,1.0f,1.0f);
                gl.glBegin(gl.GL_LINE_STRIP);
                gl.glVertex3d(0,0,0.5);
                gl.glVertex3d(0.25,0.25,0.5);
                gl.glVertex3d(0.25,-0.25,0.5);
                gl.glVertex3d(-0.25,0.25,0.5);
                gl.glVertex3d(0.25,0.25,-0.5);
                gl.glEnd();*/
        }
        
        if(measureFps) {
            endTime = System.nanoTime();
            double totalTime = (endTime - startTime)*1E-9;
            System.err.println("Time to draw:"+totalTime + " sec");
        }
        //Take screenshot AFTER the scene is draw because Screenshot class reads from current buffer (maybe can change read buffer to set it, but htis works, too)
        if (ssFile != null) {
            try {
                com.sun.opengl.util.Screenshot.writeToFile(ssFile, frameWidth, frameHeight);
            } catch (Exception e) {
                throw new SDDMException("Problem Taking Screenshot:"+e.getMessage(),e);
            }
            ssFile = null;
        }
    }
    
    public void displayChanged(GLAutoDrawable drawable, boolean modeChanged, boolean deviceChanged) {
    }
    /**
     * Returns the camera used by this listener
     * @return  the camera used in this listener
     */
    public StereoCamera getCamera() {
        return camera;
    }
    /**
     * Sets the camera for this listener
     * @param camera the camera to use in this listener
     */
    public void setCamera(StereoCamera camera) {
        this.camera = camera;
    }
    public void setAnaglyphColors(float leftRed, float leftGreen, float leftBlue, float rightRed, float rightGreen, float rightBlue) {
        this.lColorRed = leftRed;
        this.lColorGreen = leftGreen;
        this.lColorBlue = leftBlue;
        this.rColorRed = rightRed;
        this.rColorGreen = rightGreen;
        this.rColorBlue = rightBlue;
    }
    public java.awt.Color getAnaglyphLeftColor() {
        return new java.awt.Color(this.lColorRed,this.lColorGreen,this.lColorBlue);
    }
    public java.awt.Color getAnaglyphRightColor() {
        return new java.awt.Color(this.rColorRed,this.rColorGreen,this.rColorBlue);
    }
    public void setAnaglyphLeftColor(java.awt.Color leftEyeColor) {
        this.lColorRed = leftEyeColor.getRed()/255;
        this.lColorGreen = leftEyeColor.getGreen()/255;
        this.lColorBlue = leftEyeColor.getBlue()/255;
    }
    public void setAnaglyphRightColor(java.awt.Color rightEyeColor) {
        this.rColorRed = rightEyeColor.getRed()/255;
        this.rColorGreen = rightEyeColor.getGreen()/255;
        this.rColorBlue = rightEyeColor.getBlue()/255;
    }
    /**
     * This activates or deactivates debug mode - must be called before window is created, or window must be disposed and recreated to reflect these changes.
     * @param debugEnabled if true, debug mode should be activated
     */
    public void setDebugEnabled(boolean debugEnabled) {
        this.debugEnabled = debugEnabled;
    }
    
    /**
     * Determines if OpenGL debug mode is activated
     * @return true if debug mode is on
     */
    public boolean isDebugEnabled() {
        return debugEnabled;
    }
    /**
     * Tells the Listener to take a screenshot instead of display the module the next time it is redisplayed
     * @param f the file to save to (format determined by extension)
     */
    public void setScreenshot(File f) {
        this.ssFile = f;
    }
    
    public boolean isMeasureFps() {
        return measureFps;
    }
    
    public void setMeasureFps(boolean measureFps) {
        this.measureFps = measureFps;
    }
    
    public static void windowPos2iWorkaround(GL gl, int x, int y) {
        if(gl.isFunctionAvailable("glWindowPos2i"))
                                gl.glWindowPos2i(x,y);
                            else if (gl.isFunctionAvailable("glWindowPos2iARB"))
                                gl.glWindowPos2iARB(x,y);
                            else {
                                gl.glMatrixMode(GL.GL_MODELVIEW);
                                gl.glPushMatrix();
                                gl.glLoadIdentity();
                                gl.glMatrixMode(GL.GL_PROJECTION);
                                gl.glPushMatrix();
                                gl.glLoadIdentity();
                                
                                if(x==0 && y==0)
                                    gl.glRasterPos2f(-1,-1);
                                else
                                    ;//TODO:implement
                                
                                gl.glPopMatrix();
                                gl.glMatrixMode(GL.GL_MODELVIEW);
                                gl.glPopMatrix();
                            }
    }
}
