/*
 * StereoCamera.java
 *
 * Created on June 1, 2006, 7:06 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package sddm;

//import org.lwjgl.util.vector.*;
import java.awt.event.MouseEvent;
import javax.vecmath.*;
import javax.media.opengl.*;

/**
 *
 * @author Erik J. Tollerud
 */
public class StereoCamera {
    enum StereoMode{NO_STEREO,BASIC_STEREO,FRUSTUM_NO_STEREO,FRUSTUM_STEREO};
    
    private javax.media.opengl.glu.GLU glu;
    private StereoMode mode;
    private FrustumSettings fsObj;
    private Vector3f c; //Camera location
    private Vector3f u; //Up vector
    private Vector3f n; //Normal vector / view direction vector
    private Vector3f t; //Target location vector
    private boolean onRight;
    private float stereoSep;
    
    private java.awt.Color sphereColor;
    private boolean tieFocalLengthToTargetPoint;
    public float sphereSizeOnUpdate; //The size of the sphere to draw when drawSphereOnUpdate is true
    public boolean drawSphereOnUpdate; //Determines if a sphere is drawn at the end of matrix setting of the camera
    
    private EyeTransforms transforms;
    
    /** Creates a new instance of StereoCamera */
    public StereoCamera() {
        this(0,0,0,0,0,-1,0,1,0);
        mode = StereoMode.NO_STEREO;
    }
    public StereoCamera(StereoMode sm) {
        this(0,0,0,0,0,-1,0,1,0);
        mode = sm;
        
    }
    public StereoCamera(float locX, float locY, float locZ) {
        this(locX,locY,locZ,0,0,-1,0,1,0);
    }
    public StereoCamera(Vector3f location) {
        this(location.x,location.y,location.z,0,0,-1,0,1,0);
    }
    public StereoCamera(float locX, float locY, float locZ, float viewX, float viewY, float viewZ, float upX, float upY, float upZ) {
        this.glu = new javax.media.opengl.glu.GLU();
        sphereColor = new java.awt.Color(0.81f,0.71f,0.23f);
        this.onRight = false;
        this.drawSphereOnUpdate = false;
        this.tieFocalLengthToTargetPoint = false;
        this.sphereSizeOnUpdate = 1.0f;
        this.stereoSep = 0.15f;
        this.fsObj = new FrustumSettings();
        this.c = new Vector3f(locX, locY, locZ);
        this.u = new Vector3f(upX, upY, upZ);
        this.n = new Vector3f(viewX, viewY, viewZ);
        this.t = new Vector3f();
        this.transforms = null;
        setTargetDistance(1);
    }
    public StereoCamera(Vector3f location, Vector3f view, Vector3f up) {
        this(location.x,location.y,location.z,view.x,view.y,view.z,up.x,up.y,up.z);
    }
    public void updateCamera(GL g) {
        //lwjgl vector method
        /*Vector3f r = Vector3f.cross(n,u,null); //right vector */
        Vector3f r = new Vector3f();
        r.cross(n,u);
        
        float eyeFactor = onRight?1:-1;
        switch(mode) {
            case BASIC_STEREO:
                //User provides projection - modelview adjusted for camera location and direction for each eye as determined by eye seperation
                //TODO: determine if this should pre- or post-multiply, and if projection is the right place
                if (this.transforms!=null) {
                    g.glMatrixMode(g.GL_PROJECTION);
                    transforms.transform(onRight, g);
                }
                
                g.glMatrixMode(g.GL_MODELVIEW);
                g.glLoadIdentity();
                r.scale(stereoSep/2.0f);
                r.scale(eyeFactor);
                glu.gluLookAt(c.x+r.x,c.y+r.y,c.z+r.z,t.x,t.y,t.z,u.x,u.y,u.z);
                onRight = onRight?false:true;
                break;
            case FRUSTUM_STEREO:
                //uses FrustumSettings to generate "correct" stereoscopic effect
                g.glMatrixMode(g.GL_PROJECTION);
                g.glLoadIdentity();
                g.glFrustum(fsObj.getLeft()-(eyeFactor*fsObj.getLRFactor(stereoSep)),fsObj.getRight()-(eyeFactor*fsObj.getLRFactor(stereoSep)),fsObj.getBottom(),fsObj.getTop(),fsObj.getNearField(),fsObj.getFarField());
                if (this.transforms!=null) 
                    transforms.transform(onRight, g);
                
                g.glMatrixMode(g.GL_MODELVIEW);
                g.glLoadIdentity();
                r.scale(stereoSep/2.0f);
                r.scale(eyeFactor);
                //glu.gluLookAt(c.x+r.x,c.y+r.y,c.z+r.z,t.x,t.y,t.z,u.x,u.y,u.z);
                glu.gluLookAt(c.x+r.x,c.y+r.y,c.z+r.z,c.x+r.x+n.x,c.y+r.y+n.y,c.z+r.z+n.z,u.x,u.y,u.z);
                onRight = onRight?false:true;
                break;
            case FRUSTUM_NO_STEREO:
                //uses glprojection
                g.glMatrixMode(g.GL_PROJECTION);
                g.glLoadIdentity();
                glu.gluPerspective(fsObj.cameraAperture,fsObj.screenAspectRatio,fsObj.nearField,fsObj.farField);
                if (this.transforms!=null) 
                    transforms.transform(onRight, g);
                
                g.glMatrixMode(g.GL_MODELVIEW);
                g.glLoadIdentity();
                glu.gluLookAt(c.x,c.y,c.z,t.x,t.y,t.z,u.x,u.y,u.z);
                break;
            case NO_STEREO:
            default:
                //User provides projection- modelview adjusted for camera location and direction in the "centerpoint" between eyes
                if (this.transforms!=null) {
                    g.glMatrixMode(g.GL_PROJECTION);
                    transforms.transform(onRight, g);
                }
                
                g.glMatrixMode(g.GL_MODELVIEW);
                g.glLoadIdentity();
                glu.gluLookAt(c.x,c.y,c.z,t.x,t.y,t.z,u.x,u.y,u.z);
        }
        if (drawSphereOnUpdate)
            this.drawTargetSphere(g,sphereSizeOnUpdate);
    }
    public void setMode(StereoMode modeIn) {
        this.mode = modeIn;
    }
    public StereoMode getMode() {
        return mode;
    }
    public void setRight(boolean right) {
        this.onRight = right;
    }
    public boolean isRight() {
        return onRight;
    }
    public boolean isStereo() {
        return (mode == StereoMode.FRUSTUM_STEREO || mode == StereoMode.BASIC_STEREO);
    }
    public Vector3f getCameraLocation() {
        return new Vector3f(c);
    }
    public Vector3f getViewDirection() {
        return new Vector3f(n);
    }
    public Vector3f getUpVector() {
        return new Vector3f(u);
    }
    public Vector3f getTargetLocation() {
        return new Vector3f(t);
    }
    public void setTargetDistance(float distance) {
        t.set(n);
        t.scale(distance);
        t.add(c);
        if (distance < 0)
            n.negate();
        if (this.tieFocalLengthToTargetPoint)
            this.fsObj.cameraFocalLength = this.getTargetDistance();
    }
    public float getTargetDistance() {
        Vector3f diff = new Vector3f();
        diff.sub(t,c);
        return diff.length();
    }
    public void setTargetPoint(Vector3f targetPoint) {
        this.setTargetPoint(targetPoint.x,targetPoint.y,targetPoint.z);
    }
    public void setTargetPoint(float x, float y, float z) {
        t.set(x,y,z);
        //lwjgl vector method
        /*Vector3f.sub(t,c,n);*/
        n.sub(t,c);
        
        //calculate new up vector perpendicular to normal vector
        //lwjgl vector method
        /*Vector3f.cross(n,u,u);
        Vector3f.cross(u,n,u);*/
        u.cross(n,u);
        u.cross(u,n);
        
        u.normalize();
        n.normalize();
        
        if (this.tieFocalLengthToTargetPoint)
            this.fsObj.cameraFocalLength = this.getTargetDistance();
    }
    public void setCameraLocation(Vector3f location) {
        this.setCameraLocation(location.x,location.y,location.z);
    }
    public void setCameraLocation(float x, float y, float z) {
        c.set(x,y,z);
    }
    //ViewVector is dominant
    public void setViewVector(Vector3f viewDirection) {
        this.setViewVector(viewDirection.x,viewDirection.y,viewDirection.z);
    }
    public void setViewVector(float x, float y, float z) {
        n.set(x,y,z);
        //Re-align up vector to lie in viewing plane
        //lwjgl vector method
        /*Vector3f r = Vector3f.cross(n,u,null); //right vector
        Vector3f.cross(r,n,u);*/
        //right vector
        Vector3f r = new Vector3f();
        r.cross(n,u);
        u.cross(r,n);
        
        n.normalize();
        u.normalize();
        //r is now holder for subtraction to reset target point
        r.set(t);
        r.sub(c);
        this.setTargetDistance(r.length());
    }
//up vector is dominant
    public void setUpVector(Vector3f up) {
        this.setUpVector(up.x,up.y,up.z);
    }
    public void setUpVector(float x, float y, float z) {
        u.set(x,y,z);
        //Re-align viewing vector to lie normal to viewing plane
        //lwjgl vector method
        /*Vector3f r = Vector3f.cross(n,u,null); //right vector
        Vector3f.cross(u,r,n);*/
        Vector3f r = new Vector3f();
        r.cross(n,u);
        n.cross(u,r);
        
        n.normalize();
        u.normalize();
        //r is now holder for subtraction to reset target point
        r.set(t);
        r.sub(c);
        this.setTargetDistance(r.length());
    }
    /** Translates the camera along the view vector.
     *@param distance distance to translate the camera. Positive values are forward, negative values are backward.
     */
    public void translate(float distance) {
        //assumes n is normalized (as it should be) at outset
        Vector3f v = new Vector3f(n);
        v.scale(distance);
        //lwjgl vector method
        /*Vector3f.add(c,n,c);
        Vector3f.add(t,n,t);*/
        c.add(v);
        t.add(v);
    }
    /** Slews the camera along the up vector.
     *@param distance distance to translate the camera. Positive values are up, negative values are down.
     */
    public void slew(float distance) {
        //Long-term TODO: do these all in quaternions
        //assumes u is normalized (as it should be) at outset
        Vector3f v = new Vector3f(u);
        v.scale(distance);
        //lwjgl vector method
        /*Vector3f.add(c,u,c);
        Vector3f.add(t,u,t);*/
        c.add(v);
        t.add(v);
    }
    /** Strafes the camera along the right vector.
     *@param distance distance to move the camera. Positive values are right, negative values are left.
     */
    public void strafe(float distance) {
        //assumes n and u are normalized (as they should be) at outset
        //lwjgl vector method
        /*Vector3f r = Vector3f.cross(n,u,null); //right vector
        r.scale(distance);
        Vector3f.add(c,r,c);
        Vector3f.add(t,r,t);*/
        Vector3f r = new Vector3f();
        r.cross(n,u);
        r.scale(distance);
        c.add(r);
        t.add(r);
    }
    /** Roll the camera about the vieing direction axis
     *@param angle angle to roll in degrees.  Positive is clockwise, negative is counter-clockwise.
     */
    public void roll(float angle) {
        float dist = this.getTargetDistance();
        // add a scaled right vector to the up vector, then re-target
        //lwjgl vector method
        /*Vector3f r = Vector3f.cross(n,u,null); //right vector
        r.scale((float)Math.toRadians(angle));
        Vector3f.add(u,r,u);*/
        Vector3f r = new Vector3f();
        r.cross(n,u);
        r.scale((float)Math.toRadians(angle));
        u.add(r);
        
        //Must keep right vector on right for cross product to be correct - so if it was leftified, re-right it
        //TODO: make angle>90 safe?
        if(angle<0)
            r.negate();
        
        //fix normal vector
        //lwjgl vector method
        /*Vector3f.cross(u,r,n);*/
        n.cross(u,r);
        
        n.normalize();
        u.normalize();
        
        this.setTargetDistance(dist);
    }
    /**Pitch the camera; up or down from the viewer's perspective
     *@param angle angle to tilt in degrees.  Positive angles are up, negative angles downward.
     */
    public void pitch(float angle) {
        float dist = this.getTargetDistance();
        // add a scaled up vector to the normal vector then retarget
        Vector3f v = new Vector3f(u);
        v.scale((float)Math.toRadians(angle));
        //lwjgl vector method
        /*Vector3f.add(n,u,n);*/
        n.add(v);
        
        //fix up vector
        //lwjgl vector method
        /*Vector3f r = Vector3f.cross(n,u,null); //right vector
        Vector3f.cross(r,n,u);*/
        Vector3f r = new Vector3f();
        r.cross(n,u);
        u.cross(r,n);
        
        n.normalize();
        u.normalize();
        
        this.setTargetDistance(dist);
    }
    /** Rotate the camera; left or right from the viewer's perspective
     *@param angle angle to rotate in degrees.  Positive angles rotate right, negative angles rotate left. Warning: errors may occur if this is zero
     */
    public void yaw(float angle) {
        //at zero angle, funky stuff happens
        
        float dist = this.getTargetDistance();
        //subtract a scaled right vector from the normal vector then retarget
        //lwjgl vector method
        /*Vector3f r = Vector3f.cross(n,u,null); //right vector
        r.scale((float)Math.toRadians(angle));
        Vector3f.sub(n,r,n);*/
        Vector3f r = new Vector3f();
        r.cross(n,u);
        r.scale((float)Math.toRadians(angle));
        n.sub(r);
        //Must keep right vector on right for cross product to be correct - so if it was leftified, re-right it
        //TODO: make angle>90 safe?
        if(angle<0)
            r.negate();
        
        //fix up vector
        //lwjgl vector method
        /*Vector3f.cross(r,n,u);*/
        u.cross(r,n);
        
        n.normalize();
        u.normalize();
        
        this.setTargetDistance(dist);
        
    }
    /** Rotate the camera left or right around the rotation point
     *@param angle angle to rotate in degrees.  Positive angles rotate right, negative angles rotate left.
     */
    public void rotateAboutTargetPoint(float angle) {
        /*
        //subtract a scaled right vector from the camera location
        Vector3f r = Vector3f.cross(n,u,null); //right vector
        r.scale((float)Math.toRadians(angle));
        Vector3f.sub(c,r,c);
        //re-aim at the target point
        setTargetPoint(t);
         */
        /*
        //lwjgl Quaternion implementation
        Vector3f diff = Vector3f.sub(c,t,null); //vector from target point to camera location
        Quaternion v = new Quaternion();
        v.set(diff);
        Quaternion z = new Quaternion();
        //z.setFromAxisAngle(new Vector4f(u.x,u.y,u.z,(float)Math.toRadians(angle)));//rotation should be about up vector
        //perform v = z*v*z^-1:
        Quaternion.mulInverse(v,z,v);
        Quaternion.mul(z,v,v);
        //set new camera location based on rotated vector
        Vector3f vVec =  v.getImagPart();
        Vector3f.add(t,vVec,c);
        setTargetPoint(t); //Re-align to target
         */
        //vecmath Quaternion implementation
        
        
        Vector3f diff = new Vector3f();
        diff.sub(c,t); //vector from target point to camera location
        Quat4f v = new Quat4f();
        v.set(diff.x,diff.y,diff.z,0);
        Quat4f z = new Quat4f();
        z.set(new AxisAngle4f(u,(float)Math.toRadians(angle)));//rotation should be about up vector
        //perform v = z*v*z^-1:
        v.mulInverse(z);
        v.mul(z,v);
        //set new camera location based on rotated vector
        Vector3f vVec =  new Vector3f(v.x,v.y,v.z);
        c.add(t,vVec);
        setTargetPoint(t); //Re-align to target
        
    }
    /**Tilt the camera up or down around the rotation point
     *@param angle angle to tilt in degrees.  Positive angles tilt up, negative angles tilt downward.
     */
    public void tiltAboutTargetPoint(float angle) {
        /*
        // add a scaled up vector to the location vector
        u.scale((float)Math.toRadians(angle));
        Vector3f.add(c,u,c);
        //re-aim at the target point
        setTargetPoint(t);
         */
        /*
        //lwjgl Quaternion implementation
        Vector3f r = Vector3f.cross(n,u,null); //right vector
        Vector3f diff = Vector3f.sub(c,t,null); //vector from target point to camera location
        Quaternion v = new Quaternion();
        v.set(diff);
        Quaternion z = new Quaternion();
        z.setFromAxisAngle(new Vector4f(r.x,r.y,r.z,(float)Math.toRadians(angle)));//rotation should be about right vector
        //perform v = z*v*z^-1:
        Quaternion.mulInverse(v,z,v);
        Quaternion.mul(z,v,v);
        //set new camera location based on rotated vector
        Vector3f vVec =  v.getImagPart();
        Vector3f.add(t,vVec,c);
        setTargetPoint(t); //Re-align to target
         */
        //vecmath Quaternion implementation
        
        Vector3f r = new Vector3f();
        r.cross(n,u);
        Vector3f diff = new Vector3f();
        diff.sub(c,t); //vector from target point to camera location
        Quat4f v = new Quat4f();
        v.set(diff.x,diff.y,diff.z,0);
        Quat4f z = new Quat4f();
        z.set(new AxisAngle4f(r,(float)Math.toRadians(angle)));//rotation should be about right vector
        //perform v = z*v*z^-1:
        v.mulInverse(z);
        v.mul(z,v);
        //set new camera location based on rotated vector
        Vector3f vVec =  new Vector3f(v.x,v.y,v.z);
        c.add(t,vVec);
        setTargetPoint(t); //Re-align to target
        
    }
    public FrustumSettings getFrustumSettings() {
        return new FrustumSettings(this.fsObj);
    }
    public void setFrustumSettings(FrustumSettings fsIn) {
        this.fsObj = new FrustumSettings(fsIn);
    }
    public void setFrustumSettings(float screenAspectRatio, float cameraAperture, float cameraFocalLength, float nearField, float farField) {
        this.fsObj = new FrustumSettings(screenAspectRatio, cameraAperture, cameraFocalLength, nearField, farField);
    }
    public float getStereoSep() {
        return stereoSep;
    }
    public void setStereoSep(float stereoSep) {
        this.stereoSep = stereoSep;
    }
    public void drawTargetSphere(GL g, float size) {
        com.sun.opengl.util.GLUT glut = new com.sun.opengl.util.GLUT();
        
        g.glPushAttrib(g.GL_ENABLE_BIT|g.GL_LIGHTING_BIT);
        g.glDisable(g.GL_LIGHTING);
        g.glDisable(g.GL_CULL_FACE);
        g.glColor3fv(sphereColor.getRGBColorComponents(null),0);
        g.glPushMatrix();
        g.glTranslatef(t.x,t.y,t.z);
        glut.glutWireSphere(size,16,8);
        g.glPopMatrix();
        g.glPopAttrib();
    }
    public boolean isFocalLengthTiedToTargetPoint() {
        return this.tieFocalLengthToTargetPoint;
    }
    
    public void setTieFocalLengthToTargetPoint(boolean b) {
        this.tieFocalLengthToTargetPoint = b;
        if(b) 
            this.fsObj.cameraFocalLength = this.getTargetDistance();
        
    }
    
    public EyeTransforms getTransforms() {
        if (this.transforms == null)
            transforms = new EyeTransforms();
        return this.transforms;
    }
    
    public void clearTransforms() {
        this.transforms = null;
    }
    
    public java.util.EventListener getTransformListener() {
        return new TransformListener(false,this.getTransforms(), null);
    }
    
    public java.util.EventListener getTransformListener(java.awt.Component[] componentsToUpdate) {
        return new TransformListener(false,this.getTransforms(), componentsToUpdate);
    }
    
    public java.util.EventListener getTransformListener(boolean changeRight) {
        return new TransformListener(changeRight,this.getTransforms(), null);
    }
    
    public java.util.EventListener getTransformListener(boolean changeRight, java.awt.Component[] componentsToUpdate) {
        return new TransformListener(changeRight,this.getTransforms(), componentsToUpdate);
    }
    
    public String toString() {
        StringBuffer sb = new StringBuffer("Stereo Camera in ");
        sb.append(this.mode.name());
        sb.append(" mode\n");
        sb.append("Location: ");
        sb.append(this.c.toString());
        sb.append("\nView Direction: ");
        sb.append(this.n.toString());
        sb.append("\nUp Direction: ");
        sb.append(this.u.toString());
        
        return sb.toString();
    }
    
    /** Member class to encapsulate relative deformations applied differently to each eye (keystone correction, etc.)
     **/
    public class EyeTransforms {
        protected float rxangle,ryangle,rzangle;
        protected float lxangle,lyangle,lzangle;
        
        EyeTransforms() {
            rxangle=0;ryangle=0;rzangle=0;
            lxangle=0;lyangle=0;lzangle=0;
        }
        
        public void transform(boolean onRight, GL gl) {
            //TODO: test if these are really at all worthwhile
            if (onRight) {
                gl.glRotatef(rxangle,1,0,0);
                gl.glRotatef(ryangle,0,1,0);
                gl.glRotatef(rzangle,0,0,1);
            } else {
                gl.glRotatef(lxangle,1,0,0);
                gl.glRotatef(lyangle,0,1,0);
                gl.glRotatef(lzangle,0,0,1);
            }
        }
        
        public void setXAngle(boolean right, float angle) {
            if (right)
                rxangle = angle;
            else
                lxangle = angle;
        }
        
        public void setYAngle(boolean right, float angle) {
            if (right)
                ryangle = angle;
            else
                lyangle = angle;
        }
        
        public void setZAngle(boolean right, float angle) {
            if (right)
                rzangle = angle;
            else
                lzangle = angle;
        }

        public void setAngles(boolean right, float x, float y, float z) {
            if (right) {
                rxangle = x;
                ryangle = y;
                rzangle = z;
            } else {
                lxangle = x;
                lyangle = y;
                lzangle = z;
            }
        }
        
        public void setAngles(boolean right, float[] xyzangles) {
            if (right) {
                rxangle = xyzangles[0];
                ryangle = xyzangles[1];
                rzangle = xyzangles[2];
            } else {
                lxangle = xyzangles[0];
                lyangle = xyzangles[1];
                lzangle = xyzangles[2];
            }
        }
        
        public float[] getAngles(boolean right) {
            float[] ret = new float[3];
            ret[0] = right?rxangle:lxangle;
            ret[1] = right?ryangle:lyangle;
            ret[2] = right?rzangle:lzangle;
            return ret;
        }
        
    }
    
    /** Interface class to make it easier to set transforms by clicking and dragging mouse around
     */
    private enum TransformMode{XZMODE, YMODE};
    private class TransformListener implements java.awt.event.MouseListener, java.awt.event.MouseMotionListener {
        EyeTransforms trans;
        boolean changeRight;
        
        TransformMode mode;
        boolean moving;
        int lastX;
        int lastY;
        
        java.awt.Component[] compsToUpdate;
        
        TransformListener(boolean changeRightIn, EyeTransforms transIn, java.awt.Component[] compsToUpdateIn) {
            trans = transIn;
            changeRight = changeRightIn;
            mode = TransformMode.XZMODE;
            moving = false;
            compsToUpdate = compsToUpdateIn;
        }

        public void mouseReleased(MouseEvent e) {
            moving = false;
        }

        public void mousePressed(MouseEvent e) {
            moving = true;
            lastX = e.getX();
            lastY = e.getY();
            
            if ( e.getButton() == MouseEvent.BUTTON1)
                mode = TransformMode.XZMODE;
            else
                mode = TransformMode.YMODE;
            
            
        }

        public void mouseMoved(MouseEvent e) {
        }

        public void mouseExited(MouseEvent e) {
        }
        
        public void mouseEntered(MouseEvent e) {
        }
        
        public void mouseDragged(MouseEvent e) {
            int deltaX = e.getX() - lastX;
            int deltaY = e.getY() - lastY;
            
            switch (mode) {
                case XZMODE:
                    trans.lzangle+=deltaX;
                    trans.lxangle+=deltaY;
                    break;
                case YMODE:
                    trans.lyangle+=deltaX;
                    break;
            }
            
            lastX = e.getX();
            lastY = e.getY();
            
            if (compsToUpdate != null) {
                for (java.awt.Component c : this.compsToUpdate) {
                    c.repaint();
                }
            }
        }
        
        public void mouseClicked(MouseEvent e) {
            if(e.getButton() != MouseEvent.BUTTON1) {
                trans.lxangle = 0;
                trans.lyangle = 0;
                trans.lzangle = 0;
                trans.rxangle = 0;
                trans.ryangle = 0;
                trans.rzangle = 0;
                
                if (compsToUpdate != null) {
                    for (java.awt.Component c : this.compsToUpdate) {
                        c.repaint();
                    }
                }
            }
        }
        
        
    }
    
    /** Convinience class to encapsulate necessary frustum settings.  Note that there is no value-checking, so use with some care.
     */
    public static class FrustumSettings {
        public float screenAspectRatio, cameraAperture, cameraFocalLength, nearField, farField;
        FrustumSettings() {
            this.screenAspectRatio = 1;
            this.cameraAperture = 90;
            this.cameraFocalLength = 10;
            this.nearField = 0.1f;
            this.farField = 80000;
        }
        FrustumSettings(FrustumSettings fsIn) {
            this.screenAspectRatio = fsIn.screenAspectRatio;
            this.cameraFocalLength = fsIn.cameraFocalLength;
            this.cameraAperture = fsIn.cameraAperture;
            this.farField = fsIn.farField;
            this.nearField = fsIn.nearField;
        }
        FrustumSettings(float screenAspectRatio, float cameraAperture, float cameraFocalLength, float nearField, float farField) {
            this.screenAspectRatio = screenAspectRatio;
            this.cameraAperture = cameraAperture;
            this.cameraFocalLength = cameraFocalLength;
            this.nearField = nearField;
            this.farField = farField;
        }
        public float getNearField() {
            return this.nearField;
        }
        public float getFarField() {
            return this.farField;
        }
        public float getApertureInRadians() {
            return (float)Math.toRadians(this.cameraAperture/2.0f);
        };
        public float getNdfl() {
            return this.nearField/this.cameraFocalLength;
        }
        public float getTop() {
            return this.nearField*(float)Math.tan(this.getApertureInRadians());
        }
        public float getBottom() {
            return getTop()*-1.0f;
        }
        public float getRight() {
            return this.screenAspectRatio*getTop();
        }
        public float getLeft() {
            return -1.0f*getRight();
        }
        public float getLRFactor(float stereoSep) {
            return 0.5f*stereoSep*this.getNdfl();
        }
    }
}
