/*
 * StereoCameraControlFrame.java
 *
 * Created on June 8, 2006, 11:03 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package sddm;
import java.awt.event.WindowEvent;
import javax.swing.*;
import java.awt.*;
import java.util.*;
import java.io.*;
import net.java.joglutils.*;
import sddm.SDDModuleControlPanel;

/**
 *
 * @author Erik J. Tollerud
 * A Frame for controls of a set of GLJFrames with StereoCameras
 *TODO: fix close button so that only the correct module is closed
 */
public class StereoCameraControlFrame extends JFrame{
    private StereoCameraControlPanel controlPanel;
    private SDDModuleControlPanel secondPanel;
    private StereoCamera camera;
    protected ArrayList<JFrame> attachedFrames;
    
    /** Creates a new instance of StereoCameraControlFrame */
    public StereoCameraControlFrame() {
        this("Stereo Camera Control", new StereoCamera());
    }
    /**
     * Creates a new instance of StereoCameraControlFrame
     * @param title the title for the window
     */
    public StereoCameraControlFrame(String title) {
        this(title, new StereoCamera());
    }
    /**
     * Creates a new instance of StereoCameraControlFrame
     * @param title the title for the window
     * @param camera the camera to attach to this frame's camera control
     */
    public StereoCameraControlFrame(String title, StereoCamera camera) {
        super(title);
        attachedFrames = new ArrayList<JFrame>();
        this.camera = camera;
        this.setLayout(new BorderLayout());
        controlPanel = new StereoCameraControlPanel(camera);
        controlPanel.setBorder(BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED));
        this.add(controlPanel,BorderLayout.NORTH);
    }
    /**
     * Creates a new instance of StereoCameraControlFrame
     * @param secondPanel the panel to use for the lower panel
     */
    public StereoCameraControlFrame(SDDModuleControlPanel secondPanel) {
        this("Stereo Camera Control", secondPanel);
    }
    /**
     * Creates a new instance of StereoCameraControlFrame
     * @param title the title for the window
     * @param secondPanel  the panel to use for the lower panel
     */
    public StereoCameraControlFrame(String title, SDDModuleControlPanel secondPanel) {
        this(title, secondPanel, new StereoCamera());
    }
    /**
     * Creates a new instance of StereoCameraControlFrame
     *
     * @param camera The camera to attach to this frame's controls
     * @param title the title for the window
     * @param secondPanel the panel to use for the lower panel
     */
    public StereoCameraControlFrame(String title, SDDModuleControlPanel secondPanel, StereoCamera camera) {
        this(title,camera);
        if (secondPanel != null) {
            this.secondPanel = secondPanel;
            secondPanel.setCamera(camera);
            secondPanel.setBorder(BorderFactory.createBevelBorder(javax.swing.border.BevelBorder.RAISED));
            this.add(secondPanel,BorderLayout.SOUTH);
        }
    }
    /**
     * Returns the camera used by this control panel
     * @return  the camera used in this panel
     */
    public StereoCamera getCamera() {
        return camera;
    }
    /**
     * Sets the camera for this control panel
     * @param camera the camera to use in this panel
     */
    public void setCamera(StereoCamera camera) {
        this.camera = camera;
    }
    /**
     * Determines if the lower panel has been generated
     * @return  true if a second panel has been passed in
     */
    public boolean isSecondPanelActive() {
        if (secondPanel == null)
            return false;
        return true;
    }
    /**
     * Attaches a JFrame to this control frame.  attached frames are repainted on calls to redisplay()
     * @param frame the frame to attach
     */
    public void attachFrame(JFrame frame) {
        attachedFrames.add(frame);
    }
    
    /**
     * Detaches the frame from this Control Frame
     * @param frame the JFrame to detach from this control frame 
     * @return true if the frame was there
     */
    public boolean removeFrame(JFrame frame) {
        return attachedFrames.remove(frame);
    }
    
    /**
     * Generates an array from the attached GLJFrames
     * @return an array point to the attached frames
     */
    public JFrame[] getAttachedFrames() {
        JFrame[] out = new JFrame[attachedFrames.size()];
        return attachedFrames.toArray(out);
    }
    
   
    
    /**
     * repaints all the attached GLJFrames and their GLCanvases in random order.  
     * Note that this will not repaint the panels on the control frame - just openGL elements
     */
    public void redisplay() {
        int numOfFrames = attachedFrames.size();
        if (numOfFrames == 1)
            attachedFrames.get(0).repaint();
        else {
            //randomizes the order of repainting
            Random rand = new java.util.Random();
            //generates a sequential list of integers
            ArrayList<Integer> intList = new ArrayList<Integer>(numOfFrames);
            for (int i = 0; i < numOfFrames; ++i)
                intList.add(i);
            //fills intArray with randomized sequential integers
            int[] intArray = new int[numOfFrames];
            for (int i = 0; i < numOfFrames; ++i)
                intArray[i] = intList.remove(rand.nextInt(numOfFrames-i));
            
            //repaint in order specified by intArray
            for(int i : intArray)
                attachedFrames.get(i).repaint();
        }
        //TODO:test if this should be commented to prevent slowing
        this.controlPanel.repaint();
        if (this.secondPanel != null)
            this.secondPanel.repaint();
    }

     public void addKeyListenerToJFrames(java.awt.event.KeyListener el) {
         for(JFrame jf : attachedFrames)
             jf.addKeyListener(el);
     }
     public void addMouseListenerToJFrames(java.awt.event.MouseListener el) {
         for(JFrame jf : attachedFrames)
             jf.addMouseListener(el);
     }
     public void addMouseMotionListenerToJFrames(java.awt.event.MouseMotionListener el) {
         for(JFrame jf : attachedFrames)
             jf.addMouseMotionListener(el);
     }
     public void addMouseWheelListenerToJFrames(java.awt.event.MouseWheelListener el) {
         for(JFrame jf : attachedFrames)
             jf.addMouseWheelListener(el);
     }
    public StereoCameraControlPanel getControlPanel() {
        return controlPanel;
    }
    public SDDModuleControlPanel getSecondPanel() {
        return secondPanel;
    }
    /**
     * Generates a screenshot from the current GLJFrames - if there are more than 1 attached frames, the filenames are appended with -1, -2, etc.
     * @param f the base filename to save the screnshot to (extension determines format)
     */
    public void takeScreenshotFromFrames(File f) throws IOException {
        //TODO: more file checking
        if (attachedFrames.size() == 1) {
            JFrame frame = attachedFrames.get(0);
            if (frame instanceof GLJFrame)
            ((SDDMListener)((GLJFrame)frame).getGLEventListener()).setScreenshot(f);
        }
        else {
            for (int i = 0; i < attachedFrames.size(); ++i) {
                JFrame frame = attachedFrames.get(i);
                if (frame instanceof GLJFrame) {
                    SDDMListener listener = (SDDMListener)((GLJFrame)frame).getGLEventListener();
                    String[] fns = f.getName().split("\\.");
                    if (fns.length !=2)
                        throw new IOException("File name does not have extension");
                    String nameTester= f.getName();
                    String dirName = f.getParent();
                    File newFile = new File(dirName+"\\"+fns[0]+"-"+(i+1)+"."+fns[1]);
                    listener.setScreenshot(newFile);
                }
            }
        }
        this.redisplay();
    }
    
    public java.awt.event.WindowListener getCloser() {
        return new java.awt.event.WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                super.windowClosing(e);
                for(JFrame frame : attachedFrames)
                    frame.dispose();
                StereoCameraControlFrame.this.dispose();
            }
        };
        
    }
    
}
