/*
 * ParametricLSTypePanel.java
 *
 * Created on August 3, 2006, 11:40 AM
 */

package sddm.modules;

import java.util.Vector;
import sddm.*;
import sddm.data.*;

/**
 *
 * @author  Erik J. Tollerud
 */
public class ParametricLSTypePanel extends ParametricModuleSetupPanel.TypePanel {
    ParametricModuleSetupPanel parent;
    
    /** Creates new form ParametricLSTypePanel */
    public ParametricLSTypePanel(ParametricModuleSetupPanel sp) {
        super();
        this.parent = sp;
        initComponents();
    }
    
    public ParametricLSTypePanel(ParametricModuleSetupPanel sp, ParametricSystem initialSystem) {
        this(sp);
        if (initialSystem != null) {
            if (initialSystem.getGen() instanceof UniformParametricGenerator) {
                UniformParametricGenerator upg = (UniformParametricGenerator)initialSystem.getGen();
                UniformParametricGenerator.Generator gen = upg.getGen();
                if (gen instanceof LSEqnGenerator) {
                    LSEqnGenerator lsg = (LSEqnGenerator)gen;
                    this.bField.setValue(lsg.b);
                    this.rField.setValue(lsg.r);
                    this.sField.setValue(lsg.s);
                    
                    this.deltaTField.setValue(upg.getDeltat());
                    this.x0Field.setValue(upg.getX0());
                    this.y0Field.setValue(upg.getY0());
                    this.z0Field.setValue(upg.getZ0());
                    
                    this.itersField.setValue(initialSystem.getCurrentIteration());
                }
            }
        }
    }

    public sddm.data.ParametricSystem makeSystem() {
        
        float b = ((Number)this.bField.getValue()).floatValue();
        float r = ((Number)this.rField.getValue()).floatValue();
        float s = ((Number)this.sField.getValue()).floatValue();
        
        float dt = ((Number)this.deltaTField.getValue()).floatValue();
        float x0 = ((Number)this.x0Field.getValue()).floatValue();
        float y0 = ((Number)this.y0Field.getValue()).floatValue();
        float z0 = ((Number)this.z0Field.getValue()).floatValue();
        
        UniformParametricGenerator upg = new UniformParametricGenerator(dt,new LSEqnGenerator(b,r,s),x0,y0,z0,0.0f);
        
        ParametricSystem ps = new ParametricSystem(upg);
        
        int iters = ((Number)this.itersField.getValue()).intValue();
        ps.iterate(iters);
        
        return ps;
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        x0Field = new javax.swing.JFormattedTextField(java.text.NumberFormat.getInstance());
        jLabel1 = new javax.swing.JLabel();
        y0Field = new javax.swing.JFormattedTextField(java.text.NumberFormat.getInstance());
        z0Field = new javax.swing.JFormattedTextField(java.text.NumberFormat.getInstance());
        deltaTField = new javax.swing.JFormattedTextField(java.text.NumberFormat.getInstance());
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        itersField = new javax.swing.JFormattedTextField(java.text.NumberFormat.getInstance());
        jLabel4 = new javax.swing.JLabel();
        jLabel5 = new javax.swing.JLabel();
        jLabel6 = new javax.swing.JLabel();
        bField = new javax.swing.JFormattedTextField(java.text.NumberFormat.getInstance());
        rField = new javax.swing.JFormattedTextField(java.text.NumberFormat.getInstance());
        sField = new javax.swing.JFormattedTextField(java.text.NumberFormat.getInstance());

        setPreferredSize(new java.awt.Dimension(204, 131));
        x0Field.setText("num");
        x0Field.setValue(1.0f);

        jLabel1.setText("Initial Point");

        y0Field.setText("num");
        y0Field.setValue(0.0f);

        z0Field.setText("num");
        z0Field.setValue(0.0f);

        deltaTField.setText("num");
        deltaTField.setValue(0.01f);

        jLabel2.setText("deltat");

        jLabel3.setText("initial iterations");

        itersField.setText("num");
        itersField.setValue(1000);

        jLabel4.setText("b");

        jLabel5.setText("r");

        jLabel6.setText("s");

        bField.setText("num");
        bField.setValue(2.66667f);

        rField.setText("num");
        rField.setValue(30.f);

        sField.setText("num");
        sField.setValue(10.0f);

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(layout.createSequentialGroup()
                        .add(jLabel1)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(x0Field, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(y0Field, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(z0Field, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(layout.createSequentialGroup()
                        .add(jLabel4)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(bField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jLabel5)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(rField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jLabel6)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(sField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                    .add(layout.createSequentialGroup()
                        .add(jLabel2)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(deltaTField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jLabel3)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(itersField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel1)
                    .add(x0Field, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(y0Field, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(z0Field, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel4)
                    .add(bField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel5)
                    .add(rField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel6)
                    .add(sField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel2)
                    .add(deltaTField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel3)
                    .add(itersField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(45, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JFormattedTextField bField;
    private javax.swing.JFormattedTextField deltaTField;
    private javax.swing.JFormattedTextField itersField;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JFormattedTextField rField;
    private javax.swing.JFormattedTextField sField;
    private javax.swing.JFormattedTextField x0Field;
    private javax.swing.JFormattedTextField y0Field;
    private javax.swing.JFormattedTextField z0Field;
    // End of variables declaration//GEN-END:variables
    
    public static class LSEqnGenerator implements sddm.data.UniformParametricGenerator.Generator {
        private float b;
        private float r;
        private float s;
        
        private float[] holder;
        
        public LSEqnGenerator(float b, float r, float s) {
            this.b = b;
            this.r = r;
            this.s = s;
            holder = new float[4];
        }
        
        public float[] nextIteration(float lastTime, float deltat, PointList wholeList) {
            //Lorenz-Salzman equations:
            //dx/dt = s(y-x)
            //dy/dt = rx-y-xz
            //dz/dt = xy-bz
            float[] res = holder;
            float[] lastElem = wholeList.getLastPoint();
            res[3] = lastElem[3];
            
            res[0] = lastElem[0] + (this.s*(lastElem[1] -lastElem[0] ))*deltat;
            res[1] = lastElem[1] + (this.r*lastElem[0] -lastElem[1]-lastElem[0] *lastElem[2])*deltat;
            res[2] = lastElem[2] + (lastElem[0] *lastElem[1]-this.b*lastElem[2])*deltat;
            
            return res;
        }
        
        public String toString() {
            return "Lorenz-Salzman equations with b="+this.b+", r="+this.r+", s="+this.s;
        }
    }
}
