/*
 * PointDrawingModule.java
 *
 * Created on August 5, 2006, 12:29 AM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package sddm.modules;


import sddm.*;
import sddm.data.*;
import javax.media.opengl.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

/**
 *
 * @author Erik J. Tollerud
 */
public class PointDrawingModule extends SDDModule{
    protected Coloring colorer;
    protected PointList points;
    protected float elemSize;
    protected int granularity;
    protected boolean joined;
    protected boolean joinPairs;
    
    
    /** Creates a new instance of PointDrawingModule */
    public PointDrawingModule() {
        this(new PointList(), new StereoCamera());
    }
    
    public PointDrawingModule(PointList points) {
        this(points, new StereoCamera());
    }
    
    public PointDrawingModule(PointSet points) {
        this(points, new StereoCamera());
    }
    
    public PointDrawingModule(StereoCamera camera) {
        this(new PointList(), camera);
    }
    
    public PointDrawingModule(PointList points, StereoCamera camera) {
        super(camera);
        colorer = new Coloring();
        this.points = points;
        
        float biggest = Math.max(Math.abs(points.getMaxMax()),Math.abs(points.getMinMin()));
        this.cam.setCameraLocation(biggest,biggest,biggest);
        this.cam.setTargetPoint(0,0,0);
        this.elemSize = 1.0f;
        this.granularity = 1;
        this.joined = false;
        //TODO: do something with the W average to make it look right
        this.cam.setTargetPoint(points.getXAvg(),points.getYAvg(),points.getZAvg());
    }
    
    public PointDrawingModule(PointSet points, StereoCamera camera) {
        super(camera);
        colorer = new Coloring();
        this.points = points.toList();
        
        float biggest = Math.max(Math.abs(points.getMaxMax()),Math.abs(points.getMinMin()));
        this.cam.setCameraLocation(biggest,biggest,biggest);
        this.cam.setTargetPoint(0,0,0);
        this.elemSize = 1.0f;
        this.granularity = 1;
        this.joined = false;
        //TODO: do something with the W average to make it look right
        this.cam.setTargetPoint(points.getXAvg(),points.getYAvg(),points.getZAvg());
    }
    
    /**
     * Sets whether or not to draw the points with lines between them
     * @param joined points are joined if true 
     */
    public void setJoined(boolean joined) {
        setJoined(joined,false);
    }
    
    /**
     * Sets whether or not to draw the points with lines between them
     * 
     * @param joinPairs if the lines are drawn only between two points (i.e. 1-2, 3-4, 5-6, 7-8 have joins between them)
     * @param joined points are joined if true (if false, other parameter does nothing)
     */
    public void setJoined(boolean joined, boolean joinPairs) {
        this.joined = joined;
        this.joinPairs = joinPairs;
        this.recompileDisplayList();
    }
    
    /**
     * Determines if points are drawn joined together
     * @return true if the points are joined
     */
    public boolean isJoined() {
        return this.joined;
    }
    
    /**
     * Determines if points are drawn as pairs
     * @return true if the points are joined and the join mode is pair-wise
     */
    public boolean isJoinedPair() {
        return this.joined && this.joinPairs;
    }
    
    /**
     * Links this module to the supplied PointSet
     * @param set the set of points to use for drawing this module
     */
    public void setPoints(PointList set) {
        this.points = set;
        this.recompileDisplayList();
    }
    
    /**
     * Gets the object used to generate this module's points  - any changes may require a recompileDisplayList().
     * @return the set of the points this module draws
     */
    public PointList getPoints() {
        return this.points;
    }
    
    public void setColorer(Coloring colorer) {
        this.colorer = colorer;
        this.recompileDisplayList();
    }
    
    public Coloring getColorer() {
        return this.colorer;
    }
    
    public void setDrawingSize(float size) {
        this.elemSize = size;
        this.recompileDisplayList();
    }
    
    public float getDrawingSize() {
        return this.elemSize;
    }
    
    public static SDDModule makeModuleFromArgs(String modName, java.util.HashMap<String,Integer> switchMap, String[] args, StereoCamera camera) throws  ParsingException {
        //Takes module types: pd|pointdraw|pointdrawing|pointdrawer
        //accepts these switches:
        //-file|f (filename|"prompt") (three|true|t)|(four|false|f)
        //[-bgcolor] r g b
        //[-size|drawingsize] #
        //[-joined|joinpoints] [(true|t)|(false|f)]
        //[Coloring switches]
        
        if (modName.matches("pd|point|pointdraw|pointdrawing|pointdrawer")) {
            PointDrawingModule m = null;
            Integer fileIndx = switchMap.remove("-f");
            if (fileIndx == null)
                fileIndx = switchMap.remove("-file");
            if (fileIndx != null) {
                String fnStr = args[fileIndx+1];
                java.io.File file;
                if (fnStr.equalsIgnoreCase("prompt")) {
                    JFileChooser jfc = new JFileChooser();
                    jfc.setFileSelectionMode(jfc.FILES_ONLY);
                    jfc.setMultiSelectionEnabled(false);
                    
                    jfc.setDialogType(jfc.OPEN_DIALOG);
                    if(jfc.APPROVE_OPTION == jfc.showDialog(null,null))
                        file = jfc.getSelectedFile();
                    else
                        throw new ParsingException("Cancelled file dialog while generating hpoint drawer");
                    
                } else
                    file = new java.io.File(fnStr);
                
                String format = args[fileIndx+2];
                boolean three = true;
                if (format.matches("three|true|t"))
                    three = true;
                else if (format.matches("four|false|f"))
                    three = false;
                else
                    throw new ParsingException("Invalid type of point data file: "+format);
                
                try{
                    m = new PointDrawingModule(new PointList(file,three),camera);
                } catch (FileFormatException e) {
                    throw new ParsingException("File Format exception trying to generate point drawing module:"+e.getMessage(),e);
                } catch (FileNotFoundException e) {
                    throw new ParsingException("File Not Found trying to generate point drawing module:"+e.getMessage(),e);
                }
                
                Integer bgIndx = switchMap.remove("-bgcolor");
                if (bgIndx != null) {
                    try{
                        float r = Float.parseFloat(args[bgIndx+1]);
                        float g = Float.parseFloat(args[bgIndx+2]);
                        float b = Float.parseFloat(args[bgIndx+3]);
                        m.setBackgroundColor(new Color(r,g,b));
                    } catch (Exception e) {
                        throw new ParsingException("Problem setting background color in Point Drawing Module:"+e.getMessage(),e);
                    }
                }
                
                Integer psIndx = switchMap.remove("-size");
                if (psIndx == null)
                    psIndx = switchMap.remove("-drawingsize");
                if (psIndx != null) {
                    try{
                        float ptsz = Float.parseFloat(args[psIndx+1]);
                        m.setDrawingSize(ptsz);
                    } catch (Exception e) {
                        throw new ParsingException("Problem setting point size in Point Drawing Module:"+e.getMessage(),e);
                    }
                }
                
                Integer joinIndx = switchMap.remove("-joined");
                if (joinIndx == null)
                    joinIndx = switchMap.remove("-joinpoints");
                if (joinIndx != null) {
                    boolean join = true;
                    if (args.length > (joinIndx+1) && args[joinIndx+1].charAt(0) != '-') {
                        String s = args[joinIndx+1];
                        if(s.equals("true") ||  s.equals("t"))
                            join = true;
                        else if (s.equals("false") ||  s.equals("f"))
                            join = false;
                        else
                            throw new ParsingException("Unrecognized command for joining points in point drawing module:"+s);
                    }
                    
                    m.setJoined(join);
                }
                
                Coloring c = Coloring.makeColoringFromArgs(switchMap, args);
                if( c != null )
                    m.setColorer(c);
            }
            
            if (m== null)
                throw new ParsingException("No field provided for point drawing module");
            return m;
        } else
            return null;
    }
    
    public void preDraw(GL gl) {
        if (this.joined)
            gl.glLineWidth(this.elemSize);
        else
            gl.glPointSize(this.elemSize);
    }
    
    public void drawGeometry(GL gl) {
        /*//Buffer method
        java.nio.FloatBuffer buffer = com.sun.opengl.util.BufferUtil.newFloatBuffer(4*points.size());
        for(float[] f : points)
            buffer.put(f);
        buffer.flip();*/
        if (this.joined) {
            if (this.joinPairs)
                gl.glBegin(GL.GL_LINES);
            else
                gl.glBegin(GL.GL_LINE_STRIP);
        }
        else
            gl.glBegin(GL.GL_POINTS);        
        
        if (colorer.isUniform()) {
            gl.glColor4fv(colorer.getColorByValue(0.0f),0);
            //array method
            for (int i = 0; i < points.size();i+=granularity)
                gl.glVertex4fv(points.getPoint(i),0);
            /*//Buffer method
            for(int i = 0; i < points.size();++i)
                gl.glVertex4fv(buffer);*/
        } else {
            colorer.setBottom(0);
            colorer.setTop(points.size());
            
            //array method
            for (int i = 0; i < points.size();i+=granularity) {
                gl.glColor4fv(colorer.getColorByValue(i),0);
                gl.glVertex4fv(points.getPoint(i),0);
            }

            /*//Buffer method - need to be fixed for granularity
             for(int i = 0; i < points.size();++i) {
                gl.glColor4fv(colorer.getColorByValue(i),0);
                gl.glVertex4fv(buffer);
            }*/
        }
        
        
        
        gl.glEnd();
    }
    
    //use frame.add<ListenerType>ToFrames
    public void attachEventListeners(StereoCameraControlFrame frame) {
        super.attachEventListeners(frame);
        final StereoCameraControlFrame finFrame = frame;
        frame.addKeyListenerToJFrames( new java.awt.event.KeyAdapter() {
            public void keyPressed(KeyEvent e) {
                if(e.getKeyChar() == 'x') {
                    if (isDrawAxis())
                        setDrawAxis(false,0,0,0);
                    else {
                        //TODO: come up with a cleverer way to calculate this
                        float xRange = points.getXMax() - points.getXMin();
                        float yRange = points.getYMax() - points.getYMin();
                        float zRange = points.getZMax() - points.getZMin();
                        setDrawAxis(true,xRange,yRange,zRange);
                    }
                }
                finFrame.redisplay();
                finFrame.repaint();
            }
        });
    }
    
    public float getMinFeatureSizeHint() {
        if (points == null)
            return super.getMinFeatureSizeHint();
        
        float xRange = this.points.getXMax() - this.points.getXMin();
        float yRange = this.points.getYMax() - this.points.getYMin();
        float zRange = this.points.getZMax() - this.points.getZMin();
        float wRange = this.points.getWMax() - this.points.getWMin();
        
        float biggestRange = Math.max(xRange,Math.max(yRange,Math.max(zRange,wRange)));
        //TODO: work out a clever way to do this - have stats get the minimum spacing?
        return biggestRange/1000.0f;
    }
    
    public static SDDModuleSetupPanel newStartupPanel() {
        return new SDDModuleSetupPanel() {
            JCheckBox threeElementCheckBox;
            JTextField fileField;
            JLabel fileLabel;
            JPanel southPanel;
            JButton browseButton;
            //Constructor
            {
                setLayout(new java.awt.BorderLayout());
                threeElementCheckBox = new JCheckBox("Three-Component File Format?");
                threeElementCheckBox.setSelected(true);
                fileLabel = new JLabel("File");
                fileField = new JTextField(40);
                browseButton = new JButton("Browse...");
                southPanel = new JPanel();
                
                
                
                add(threeElementCheckBox,java.awt.BorderLayout.NORTH);
                add(fileLabel, java.awt.BorderLayout.WEST);
                add(fileField, java.awt.BorderLayout.CENTER);
                add(southPanel,java.awt.BorderLayout.SOUTH);
                
                final JPanel parentRef = this;
                browseButton.addActionListener( new ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        JFileChooser jfc = new JFileChooser();
                        jfc.setMultiSelectionEnabled(false);
                        if(jfc.showOpenDialog(parentRef) == jfc.APPROVE_OPTION)
                            fileField.setText(jfc.getSelectedFile().toString());
                        
                    }
                    
                });
                
                southPanel.add(browseButton);
                
            }
            public SDDModule makeModule(StereoCamera camera) {
                boolean three = threeElementCheckBox.isSelected();
                File pointFile = new File(fileField.getText());
                PointList pl;
                try {
                    pl = new PointList(pointFile,three);
                } catch (FileNotFoundException e) {
                    JOptionPane.showMessageDialog(null,"File not found generating Point Drawing Module", "File not Found",JOptionPane.ERROR_MESSAGE);
                    pl = new PointList();
                } catch (FileFormatException e) {
                    JOptionPane.showMessageDialog(null,"File format incorrect generating Point Drawing Module\n"+e.getMessage(), "File not Found",JOptionPane.ERROR_MESSAGE);
                    pl = new PointList();
                }
                
                return new PointDrawingModule(pl, camera);
            }
            
        };
    }
    
    public SDDModuleControlPanel newModuleControlPanel() {
        return new SDDModuleControlPanel(PointDrawingModule.this.cam) {
            JButton colorerButton;
            JButton bgColorButton;
            JLabel psLabel;
            JFormattedTextField sizeField;
            JLabel granLabel;
            JFormattedTextField granField;
            JCheckBox joinCheckBox;
            JCheckBox pairCheckBox;
            
            
            //Constructor
            {
                colorerButton = new JButton("Point Colorer...");
                final JPanel parentRef = this; // For telling the buttons who their parent is
                colorerButton.addActionListener(new java.awt.event.ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        Coloring c = Coloring.coloringOptionPane(parentRef,PointDrawingModule.this.getColorer());
                        if (c!=null) {
                            PointDrawingModule.this.setColorer(c);
                            recompileDisplayList();
                            redisplayParent();
                        }
                    }
                });
                
                bgColorButton = new JButton("Background Color...");
                bgColorButton.addActionListener(new java.awt.event.ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        JColorChooser jcc = new JColorChooser();
                        Color res = jcc.showDialog(parentRef,"title",getBackgroundColor());
                        if (res != null)
                            setBackgroundColor(res);
                        recompileDisplayList();
                        redisplayParent();
                    }
                });
                
                psLabel = new JLabel("Size");
                sizeField = new JFormattedTextField((java.text.Format)java.text.NumberFormat.getInstance());
                sizeField.setValue(elemSize);
                sizeField.addPropertyChangeListener( new java.beans.PropertyChangeListener() {
                    public void propertyChange(java.beans.PropertyChangeEvent evt) {
                        if(evt.getPropertyName().equals("value")) {
                            setDrawingSize(((Number)sizeField.getValue()).floatValue());
                            recompileDisplayList();
                            redisplayParent();
                        }
                    }
                });
                
                granLabel = new JLabel("Granularity");
                granField = new JFormattedTextField((java.text.Format)java.text.NumberFormat.getIntegerInstance());
                granField.setValue(granularity);
                granField.addPropertyChangeListener( new java.beans.PropertyChangeListener() {
                    public void propertyChange(java.beans.PropertyChangeEvent evt) {
                        if(evt.getPropertyName().equals("value")) {
                            setGranularity(((Number)granField.getValue()).intValue());
                            recompileDisplayList();
                            redisplayParent();
                        }
                    }
                });
                
                joinCheckBox = new JCheckBox("Join points?");
                joinCheckBox.setSelected(joined);
                joinCheckBox.addActionListener(new java.awt.event.ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        setJoined(joinCheckBox.isSelected());
                        redisplayParent();
                    }
                });
                
                pairCheckBox = new JCheckBox("Pairwise?");
                pairCheckBox.setSelected(joinPairs);
                pairCheckBox.addActionListener(new java.awt.event.ActionListener() {
                    public void actionPerformed(ActionEvent e) {
                        setJoined(joinCheckBox.isSelected(),pairCheckBox.isSelected());
                        redisplayParent();
                    }
                });
                
                add(colorerButton);
                add(bgColorButton);
                add(psLabel);
                add(sizeField);
                add(granLabel);
                add(granField);
                add(joinCheckBox);
                add(pairCheckBox);
            }
            
            public SDDModule panelToModuleSettings(SDDModule module) {
                PointDrawingModule pdm = (PointDrawingModule) module;
                pdm.setDrawingSize(((Number)sizeField.getValue()).floatValue());
                //TODO:implement
                return (SDDModule)pdm;
            }
            
            public void moduleSettingsToPanel(SDDModule module) {
                PointDrawingModule pdm = (PointDrawingModule) module;
                sizeField.setValue(pdm.elemSize);
                //TODO:implement
            }
        };
    }
    
    public static String moduleTitle() {
        return "Point Drawing Module";
    }
    
    public void setGranularity(int granularity) {
        if (granularity > 1)
            this.granularity = granularity;
        else
            this.granularity = 1;
    }
    
    public int getGranularity() {
        return granularity;
    }
    
    /**
     * Saves the points to a file with each point on a line, components seperated by a space. This will append to any existing file.
     * 
     * @param saveW if true, the w-component is included
     * @param f the file in which to save the point data
     * @throws java.io.IOException if there is a problem writing to the file
     */
    public void savePoints(File f, boolean saveW) throws IOException {
        if(!f.exists())
            f.createNewFile();
        
        FileWriter fw = new FileWriter(f);
        for(float[] pt : this.points) {
            fw.write(Float.toString(pt[0]));
            fw.write(' ');
            fw.write(Float.toString(pt[1]));
            fw.write(' ');
            fw.write(Float.toString(pt[2]));
            fw.write(' ');
            if (saveW) {
                fw.write(Float.toString(pt[3]));
                fw.write(' ');
            }
            
            fw.write('\n');
        }
        fw.close();
        
    }
    
    /**
     * Saves the points to a file with each point on a line, components seperated by a space with only x,y, and z components. This will append to any existing file.
     * 
     * @param f the file in which to save the point data
     * @throws java.io.IOException if there is a problem writing to the file
     */
    public void savePoints(File f) throws IOException  {
        savePoints(f,false);
    }
    
}
