/*
 * Scalar3DModuleControlPanel.java
 *
 * Created on August 23, 2006, 1:15 PM
 */

package sddm.modules;

import sddm.*;
import net.java.joglutils.*;
import net.java.joglutils.lighting.*;
import javax.swing.*;
import java.awt.*;
/**
 *
 * @author  Erik J. Tollerud
 */
public class Scalar3DModuleControlPanel extends SDDModuleControlPanel {
    private Scalar3DModule mod;
    private sddm.data.PointMap<Number> resetToMap;
    
    /** Creates new form Scalar3DModuleControlPanel */
    public Scalar3DModuleControlPanel(Scalar3DModule mod) {
        this.mod = mod;
        resetToMap = null;
        initComponents();
    }
    
    public SDDModule panelToModuleSettings(SDDModule module) {
        if (!(module instanceof Scalar3DModule))
            throw new SDDMException("Tried to set module settings from a non-matched panel");
        
        Scalar3DModule s3m = (Scalar3DModule) module;
        s3m.setGranularity(((Number)this.granField.getValue()).intValue());
        s3m.setDrawPolygons(this.polyCheckBox.isSelected());
        s3m.setDrawOutline(this.outlineCheckBox.isSelected());
        s3m.setStipple(this.stippleCheckBox.isSelected());
        s3m.setLightingOn(this.lightingCheckBox.isSelected());
        s3m.setLightOutline(this.lightOlCheckBox.isSelected());
        s3m.getLight().setPhongShaded(this.phongCheckBox.isSelected());
        s3m.setShininess(((Number)this.shininessField.getValue()).floatValue());
        s3m.setGlobalScale(((Number)this.scaleField.getValue()).floatValue());
        s3m.setLineWidth(((Number)this.lineWidthField.getValue()).floatValue());
        s3m.setZeroBaseScaling(this.zeroScaleCheckBox.isSelected());
        
        //Set model
        selectModelFromComboBox(s3m);
        selectScalingFromComboBox(s3m);
        
        //selection setting
        if(selectCheckBox.isSelected()) {
            JFrame[] frames = this.getControlFrame().getAttachedFrames();
            for (JFrame frame: frames) {
                if (frame instanceof GLJFrame) {
                    s3m.setSelecting(((GLJFrame)frame).getAutoDrawable());
                    break;
                }
            }
        } else
            s3m.setSelecting(null);
        
        return s3m;
        
    }
    
    public void moduleSettingsToPanel(SDDModule module) {
        if (module != null) {
            if (!(module instanceof Scalar3DModule))
                throw new SDDMException("Tried to set module settings from a non-matched panel");
            
            Scalar3DModule s3m = (Scalar3DModule) module;
            this.granField.setValue(s3m.getGranularity());
            this.polyCheckBox.setSelected(s3m.isDrawPolygons());
            this.outlineCheckBox.setSelected(s3m.isDrawOutline());
            this.stippleCheckBox.setSelected(s3m.isStipple());
            this.lightingCheckBox.setSelected(s3m.isLightingOn());
            this.lightOlCheckBox.setSelected(s3m.isLightOutline());
            this.phongCheckBox.setSelected(s3m.getLight().isPhongShaded());
            this.shininessField.setValue(s3m.getShininess());
            this.scaleField.setValue(s3m.getGlobalScale());
            this.lineWidthField.setValue(s3m.getLineWidth());
            this.zeroScaleCheckBox.setSelected(s3m.isZeroBaseScaling());
            if (s3m.isSelecting()) {
                this.selectCheckBox.setSelected(true);
                this.selectTextFieldActionPerformed(new java.awt.event.ActionEvent(this.selectTextField,0,""));
            }
            else
                this.selectCheckBox.setSelected(false);
            
            selectComboBoxFromModule(s3m);
            selectScalingFromModule(s3m);
        }
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        outlineColorButton = new javax.swing.JButton();
        outlineCheckBox = new javax.swing.JCheckBox();
        polyCheckBox = new javax.swing.JCheckBox();
        lightingButton = new javax.swing.JButton();
        jLabel2 = new javax.swing.JLabel();
        shininessField = new javax.swing.JFormattedTextField(java.text.NumberFormat.getInstance());
        lightingCheckBox = new javax.swing.JCheckBox();
        bgColorButton = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();
        granField = new javax.swing.JFormattedTextField(java.text.NumberFormat.getInstance());
        stippleCheckBox = new javax.swing.JCheckBox();
        polyColorButton = new javax.swing.JButton();
        phongCheckBox = new javax.swing.JCheckBox();
        scaleField = new javax.swing.JFormattedTextField(java.text.NumberFormat.getInstance());
        jLabel3 = new javax.swing.JLabel();
        lightOlCheckBox = new javax.swing.JCheckBox();
        modelComboBox = new javax.swing.JComboBox();
        perturbButton = new javax.swing.JButton();
        lineWidthField = new javax.swing.JFormattedTextField(java.text.NumberFormat.getInstance());
        jLabel4 = new javax.swing.JLabel();
        resetButton = new javax.swing.JButton();
        zeroScaleCheckBox = new javax.swing.JCheckBox();
        selectCheckBox = new javax.swing.JCheckBox();
        selectTextField = new javax.swing.JTextField();
        targCenterButton = new javax.swing.JButton();
        scalingComboBox = new javax.swing.JComboBox();
        jLabel5 = new javax.swing.JLabel();

        setOpaque(false);
        setPreferredSize(new java.awt.Dimension(358, 249));
        outlineColorButton.setText("Outline Color Scheme...");
        outlineColorButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                outlineColorButtonActionPerformed(evt);
            }
        });

        outlineCheckBox.setSelected(mod.isDrawOutline());
        outlineCheckBox.setText("show Outline?");
        outlineCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        outlineCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        outlineCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                outlineCheckBoxActionPerformed(evt);
            }
        });

        polyCheckBox.setSelected(mod.isDrawPolygons());
        polyCheckBox.setText("show Polygon mesh?");
        polyCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        polyCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        polyCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                polyCheckBoxActionPerformed(evt);
            }
        });

        lightingButton.setText("Configure Light...");
        lightingButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                lightingButtonActionPerformed(evt);
            }
        });

        jLabel2.setText("shininess:");

        shininessField.setText("num");
        shininessField.setValue(mod.getShininess());
        shininessField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                shininessFieldActionPerformed(evt);
            }
        });

        lightingCheckBox.setSelected(this.mod.isLightingOn());
        lightingCheckBox.setText("enable Lighting");
        lightingCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        lightingCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        lightingCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                lightingCheckBoxActionPerformed(evt);
            }
        });

        bgColorButton.setText("Background Color...");
        bgColorButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bgColorButtonActionPerformed(evt);
            }
        });

        jLabel1.setText("Granularity:");

        granField.setText("num");
        granField.setValue(mod.getGranularity());
        granField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                granFieldActionPerformed(evt);
            }
        });

        stippleCheckBox.setSelected(mod.isStipple());
        stippleCheckBox.setText("sTipple outline?");
        stippleCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        stippleCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        stippleCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                stippleCheckBoxActionPerformed(evt);
            }
        });

        polyColorButton.setText("Polygon Color Scheme...");
        polyColorButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                polyColorButtonActionPerformed(evt);
            }
        });

        phongCheckBox.setSelected(this.mod.getLight().isPhongShaded());
        phongCheckBox.setText("Phong lighting");
        phongCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        phongCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        phongCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                phongCheckBoxActionPerformed(evt);
            }
        });

        scaleField.setText("num");
        scaleField.setValue(mod.getGlobalScale());
        scaleField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                scaleFieldActionPerformed(evt);
            }
        });

        jLabel3.setText("scale:");

        lightOlCheckBox.setSelected(mod.isLightOutline());
        lightOlCheckBox.setText("light outline?");
        lightOlCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        lightOlCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        lightOlCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                lightOlCheckBoxActionPerformed(evt);
            }
        });

        modelComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Sphere Model", "Cone Model", "Cube Model", "Teapot Model", "Torus Model", "Dodecahedron Model", "Icosahedron Model", "Octahedron Model", "Tetrahedron Model", "Custom Model" }));
        selectComboBoxFromModule(this.mod);
        modelComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                modelComboBoxActionPerformed(evt);
            }
        });

        perturbButton.setText("Random Perturbation...");
        perturbButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                perturbButtonActionPerformed(evt);
            }
        });

        lineWidthField.setText("num");
        lineWidthField.setValue(mod.getLineWidth());
        lineWidthField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                lineWidthFieldActionPerformed(evt);
            }
        });

        jLabel4.setText("Line width:");

        resetButton.setText("Reset");
        resetButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                resetButtonActionPerformed(evt);
            }
        });

        zeroScaleCheckBox.setSelected(mod.isZeroBaseScaling());
        zeroScaleCheckBox.setText("zero-base scaling");
        zeroScaleCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        zeroScaleCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        zeroScaleCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                zeroScaleCheckBoxActionPerformed(evt);
            }
        });

        selectCheckBox.setText("select?");
        selectCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        selectCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        selectCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                selectCheckBoxActionPerformed(evt);
            }
        });

        selectTextField.setEditable(false);
        selectTextField.setText("N/A");
        selectTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                selectTextFieldActionPerformed(evt);
            }
        });

        targCenterButton.setText("target center");
        targCenterButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                targCenterButtonActionPerformed(evt);
            }
        });

        scalingComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Linear", "Logarithmic", "Exponential", "Polynomial" }));
        scalingComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                scalingComboBoxActionPerformed(evt);
            }
        });

        jLabel5.setText("scaling method:");

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(layout.createSequentialGroup()
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(layout.createSequentialGroup()
                                .add(polyColorButton)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(polyCheckBox))
                            .add(layout.createSequentialGroup()
                                .add(jLabel1)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(granField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(perturbButton)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(resetButton))
                            .add(layout.createSequentialGroup()
                                .add(lightingButton)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(lightingCheckBox)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(phongCheckBox))
                            .add(layout.createSequentialGroup()
                                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(outlineColorButton)
                                    .add(bgColorButton))
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(layout.createSequentialGroup()
                                        .add(stippleCheckBox)
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                        .add(lightOlCheckBox))
                                    .add(layout.createSequentialGroup()
                                        .add(outlineCheckBox)
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                        .add(jLabel4)
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                        .add(lineWidthField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))))
                            .add(layout.createSequentialGroup()
                                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(layout.createSequentialGroup()
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                        .add(jLabel2)
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                        .add(shininessField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                        .add(modelComboBox, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                                    .add(layout.createSequentialGroup()
                                        .add(72, 72, 72)
                                        .add(jLabel5)
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                        .add(scalingComboBox, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)))
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(zeroScaleCheckBox)
                                    .add(targCenterButton))))
                        .addContainerGap())
                    .add(layout.createSequentialGroup()
                        .add(selectCheckBox)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(selectTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 189, Short.MAX_VALUE)
                        .add(102, 102, 102))
                    .add(layout.createSequentialGroup()
                        .add(jLabel3)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(scaleField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addContainerGap(280, Short.MAX_VALUE))))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel1)
                    .add(granField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(perturbButton)
                    .add(resetButton))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(polyColorButton)
                    .add(polyCheckBox))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(outlineColorButton)
                    .add(outlineCheckBox)
                    .add(jLabel4)
                    .add(lineWidthField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(bgColorButton)
                    .add(stippleCheckBox)
                    .add(lightOlCheckBox))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lightingButton)
                    .add(lightingCheckBox)
                    .add(phongCheckBox))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(shininessField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel2)
                    .add(modelComboBox, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(targCenterButton))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(scaleField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel3)
                    .add(jLabel5)
                    .add(scalingComboBox, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(zeroScaleCheckBox))
                .add(12, 12, 12)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(selectCheckBox)
                    .add(selectTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents

    private void scalingComboBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_scalingComboBoxActionPerformed
        try {
            selectScalingFromComboBox(this.mod);
            mod.recompileDisplayList();
            this.redisplayParent();
        } catch (Exception e) {
            JOptionPane.showMessageDialog(this,"Problem setting Scaling:"+e.getMessage(),"Scaling Error",JOptionPane.ERROR_MESSAGE);
            selectScalingFromModule(this.mod);
        }
    }//GEN-LAST:event_scalingComboBoxActionPerformed

    private void targCenterButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_targCenterButtonActionPerformed
        sddm.data.PointMap<Number> map = mod.getMap();
        cam.setTargetPoint(map.getXAvg(),map.getYAvg(),map.getZAvg());
        this.redisplayParent();
    }//GEN-LAST:event_targCenterButtonActionPerformed
        
    java.text.DecimalFormat idm = null;
    java.text.DecimalFormat vdm = null;
    private void selectTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_selectTextFieldActionPerformed
        
        if(mod.isSelecting()) {
            if (idm == null) {
                idm = new java.text.DecimalFormat();
                vdm = new java.text.DecimalFormat();
                idm.applyPattern("#####0.###");
                vdm.applyPattern("0.####E0");
            }
            
            selectTextField.setText(mod.getSelectionInfoString(idm,vdm," "));
                 
            //TODO:more intelligent field sizing
            /*if (!selectTextField.getText().equals("Invalid Selection")){
                if (selectTextField.getPreferredSize().width > selectTextField.getWidth()) {
                    while(selectTextField.getPreferredSize().width > selectTextField.getWidth()) {
                        vdm.setMaximumFractionDigits(vdm.getMaximumFractionDigits()-1);
                        selectTextField.setText(mod.getSelectionInfoString(idm,vdm," "));
                    }
                    
                }
                if (selectTextField.getPreferredSize().width < selectTextField.getWidth()) {
                    while(selectTextField.getPreferredSize().width < selectTextField.getWidth()) {
                        vdm.setMaximumFractionDigits(vdm.getMaximumFractionDigits()+1);
                        selectTextField.setText(mod.getSelectionInfoString(idm,vdm," "));
                    }
                }
                vdm.setMaximumFractionDigits(vdm.getMaximumFractionDigits()-1);
                selectTextField.setText(mod.getSelectionInfoString(idm,vdm," "));
            }*/
        } else
            selectTextField.setText("N/A");
    }//GEN-LAST:event_selectTextFieldActionPerformed
    
    private void selectCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_selectCheckBoxActionPerformed
        javax.media.opengl.GLAutoDrawable toApply = null;
        if(selectCheckBox.isSelected()) {
            JFrame[] frames = this.getControlFrame().getAttachedFrames();
            for (JFrame frame: frames) {
                if (frame instanceof GLJFrame) {
                    toApply = ((GLJFrame)frame).getAutoDrawable();
                    break;
                }
            }
        }
        //else leave null
        mod.setSelecting(toApply);
        selectCheckBox.setSelected(toApply != null);
        selectTextFieldActionPerformed(evt);
    }//GEN-LAST:event_selectCheckBoxActionPerformed
    
    private void zeroScaleCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_zeroScaleCheckBoxActionPerformed
        mod.setZeroBaseScaling(this.zeroScaleCheckBox.isSelected());
        this.redisplayParent();
    }//GEN-LAST:event_zeroScaleCheckBoxActionPerformed
    
    private void resetButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_resetButtonActionPerformed
        if (this.resetToMap != null) {
            mod.setMap(this.resetToMap);
            this.resetToMap = null;
            mod.recompileDisplayList();
            this.redisplayParent();
        }
    }//GEN-LAST:event_resetButtonActionPerformed
    
    private void modelComboBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_modelComboBoxActionPerformed
        selectModelFromComboBox(this.mod);
        mod.recompileDisplayList();
        this.redisplayParent();
    }//GEN-LAST:event_modelComboBoxActionPerformed
    
    private void lineWidthFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_lineWidthFieldActionPerformed
        mod.setLineWidth(((Number)this.lineWidthField.getValue()).floatValue());
        mod.recompileDisplayList();
        this.redisplayParent();
    }//GEN-LAST:event_lineWidthFieldActionPerformed
    
    private void perturbButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_perturbButtonActionPerformed
        String result = JOptionPane.showInputDialog(null,"Maximum amount of Perturbation?","0.1");
        try {
            float scale = Float.parseFloat(result);
            if (resetToMap == null)
                resetToMap = mod.getMap().clone();
            mod.getMap().randomPerturbation(scale,false);
            mod.recompileDisplayList();
            this.redisplayParent();
        } catch (Exception e) {
            JOptionPane.showMessageDialog(null,"Problem performing perturbation:"+e.getMessage(),"Did not perturb",JOptionPane.ERROR_MESSAGE);
            //just do nothing if there's a problem here
        }
    }//GEN-LAST:event_perturbButtonActionPerformed
    
    private void lightOlCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_lightOlCheckBoxActionPerformed
        mod.setLightOutline(lightOlCheckBox.isSelected());
        mod.recompileDisplayList();
        this.redisplayParent();
    }//GEN-LAST:event_lightOlCheckBoxActionPerformed
    
    private void scaleFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_scaleFieldActionPerformed
        float scale = ((Number)this.scaleField.getValue()).floatValue();
        if (scale < 0 ) {
            mod.setGlobalScaleToFillBox(-scale);
            selectScalingFromModule(mod);
            this.scaleField.setText(Float.toString(mod.getGlobalScale()));
        }
        else
            mod.setGlobalScale(scale);
        
        mod.recompileDisplayList();
        this.redisplayParent();
    }//GEN-LAST:event_scaleFieldActionPerformed
    
    private void outlineColorButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_outlineColorButtonActionPerformed
        Coloring c = Coloring.coloringOptionPane(this);
        
        if (c!=null) {
            mod.setOutlineColoring(c);
            mod.recompileDisplayList();
            this.redisplayParent();
        }
    }//GEN-LAST:event_outlineColorButtonActionPerformed
    
    private void polyColorButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_polyColorButtonActionPerformed
        Coloring c = Coloring.coloringOptionPane(this);
        
        if (c!=null) {
            mod.setPolygonColoring(c);
            mod.recompileDisplayList();
            this.redisplayParent();
        }
    }//GEN-LAST:event_polyColorButtonActionPerformed
    
    private void phongCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_phongCheckBoxActionPerformed
        try {
            mod.getLight().setPhongShaded(this.phongCheckBox.isSelected());
            
            mod.recompileDisplayList();
            this.redisplayParent();
        } catch(LightingException e) {
            JOptionPane.showMessageDialog(null,"Problem setting Phong shader: "+e.getMessage(),"Phong Shading Failed",JOptionPane.WARNING_MESSAGE);
        }
    }//GEN-LAST:event_phongCheckBoxActionPerformed
    
    private void stippleCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stippleCheckBoxActionPerformed
        mod.setStipple(stippleCheckBox.isSelected());
        this.redisplayParent();
    }//GEN-LAST:event_stippleCheckBoxActionPerformed
    
    private void granFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_granFieldActionPerformed
        mod.setGranularity(((Number)granField.getValue()).intValue());
        mod.recompileDisplayList();
        this.redisplayParent();
    }//GEN-LAST:event_granFieldActionPerformed
    
    private void bgColorButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bgColorButtonActionPerformed
        Color selectedColor = new JColorChooser().showDialog(this,"Background Color?",mod.getBackgroundColor());
        if (selectedColor != null) {
            mod.setBackgroundColor(selectedColor);
            this.redisplayParent();
        }
    }//GEN-LAST:event_bgColorButtonActionPerformed
    
    private void lightingCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_lightingCheckBoxActionPerformed
        this.mod.setLightingOn(lightingCheckBox.isSelected());
        this.redisplayParent();
    }//GEN-LAST:event_lightingCheckBoxActionPerformed
    
    private void shininessFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_shininessFieldActionPerformed
        try {
            mod.setShininess(((Number)shininessField.getValue()).floatValue());
            this.redisplayParent();
        } catch (SDDMException e) {
            JOptionPane.showMessageDialog(null,"Tried to set shininess greater than 128 or less than 0.  OpenGL Shininesss must be between 0 and 128, inclusive.","Couldn't set shininess",JOptionPane.ERROR_MESSAGE);
            shininessField.setValue(mod.getShininess());
        }
    }//GEN-LAST:event_shininessFieldActionPerformed
    
    private void lightingButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_lightingButtonActionPerformed
        LightPanel lp = new LightPanel(mod.getLight());
        JPanelDialog jpd = new JPanelDialog(lp);
        ColorButton cb = new ColorButton("Global Ambient");
        cb.setColor(mod.getGlobalAmbient());
        jpd.add(cb);
        if(jpd.showAsModal()) {
            lp.panelToLight(mod.getLight());
            mod.setGlobalAmbient(cb.getColor());
            if (mod.isLightingOn()) {
                mod.recompileDisplayList();
                this.redisplayParent();
            }
        }
    }//GEN-LAST:event_lightingButtonActionPerformed
    
    private void polyCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_polyCheckBoxActionPerformed
        mod.setDrawPolygons(polyCheckBox.isSelected());
        mod.recompileDisplayList();
        this.redisplayParent();
    }//GEN-LAST:event_polyCheckBoxActionPerformed
    
    private void outlineCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_outlineCheckBoxActionPerformed
        mod.setDrawOutline(outlineCheckBox.isSelected());
        mod.recompileDisplayList();
        this.redisplayParent();
    }//GEN-LAST:event_outlineCheckBoxActionPerformed
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton bgColorButton;
    private javax.swing.JFormattedTextField granField;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JCheckBox lightOlCheckBox;
    private javax.swing.JButton lightingButton;
    private javax.swing.JCheckBox lightingCheckBox;
    private javax.swing.JFormattedTextField lineWidthField;
    private javax.swing.JComboBox modelComboBox;
    private javax.swing.JCheckBox outlineCheckBox;
    private javax.swing.JButton outlineColorButton;
    private javax.swing.JButton perturbButton;
    private javax.swing.JCheckBox phongCheckBox;
    private javax.swing.JCheckBox polyCheckBox;
    private javax.swing.JButton polyColorButton;
    private javax.swing.JButton resetButton;
    private javax.swing.JFormattedTextField scaleField;
    private javax.swing.JComboBox scalingComboBox;
    private javax.swing.JCheckBox selectCheckBox;
    private javax.swing.JTextField selectTextField;
    private javax.swing.JFormattedTextField shininessField;
    private javax.swing.JCheckBox stippleCheckBox;
    private javax.swing.JButton targCenterButton;
    private javax.swing.JCheckBox zeroScaleCheckBox;
    // End of variables declaration//GEN-END:variables
/*public void paint(java.awt.Graphics g) {
        this.moduleSettingsToPanel(mod);
        super.paint(g);
    }*/    

    //-----------------------------------------privates below here------------------------------
    
    private void selectComboBoxFromModule(Scalar3DModule s3m){
        java.awt.event.ActionListener[] als = this.modelComboBox.getActionListeners();
        for (java.awt.event.ActionListener l : als)
            this.modelComboBox.removeActionListener(l);
        
        switch(s3m.getModelType()) {
            case SPHERE:
                this.modelComboBox.setSelectedIndex(0);
                break;
            case CONE:
                this.modelComboBox.setSelectedIndex(1);
                break;
            case CUBE:
                this.modelComboBox.setSelectedIndex(2);
                break;
            case TEAPOT:
                this.modelComboBox.setSelectedIndex(3);
                break;
            case TORUS:
                this.modelComboBox.setSelectedIndex(4);
                break;
            case DODECAHEDRON:
                this.modelComboBox.setSelectedIndex(5);
                break;
            case ICOSAHEDRON:
                this.modelComboBox.setSelectedIndex(6);
                break;
            case OCTAHEDRON:
                this.modelComboBox.setSelectedIndex(7);
                break;
            case TETRAHEDRON:
                this.modelComboBox.setSelectedIndex(8);
                break;
            case CUSTOM:
                this.modelComboBox.setSelectedIndex(9);
        }
        
        for (java.awt.event.ActionListener l : als)
            this.modelComboBox.addActionListener(l);
    }
    
    private void selectModelFromComboBox(Scalar3DModule s3m){
        switch(this.modelComboBox.getSelectedIndex()) {
            case 0:
                //Sphere
                String slices = null;
                String stacks = null;
                slices = JOptionPane.showInputDialog("Slices","8");
                if (slices != null)
                    stacks = JOptionPane.showInputDialog("Stacks","8");
                
                try{
                    s3m.useSphereModel(Integer.parseInt(slices),Integer.parseInt(stacks));
                } catch (Exception e){
                    JOptionPane.showMessageDialog(null,"Problem Changing to Sphere Model:"+e.getMessage(),"Couldn't change to Sphere Model",JOptionPane.WARNING_MESSAGE);
                    selectComboBoxFromModule(s3m);
                }
                break;
            case 1:
                //Cone
                String bthr = null;
                slices = null;
                stacks = null;
                
                bthr = JOptionPane.showInputDialog("Base/Height Ratio","0.5");
                if (bthr != null)
                    slices = JOptionPane.showInputDialog("Slices","8");
                if (slices != null)
                    stacks = JOptionPane.showInputDialog("Stacks","8");
                
                try{
                    s3m.useConeModel(Double.parseDouble(bthr), Integer.parseInt(slices),Integer.parseInt(stacks));
                } catch (Exception e){
                    JOptionPane.showMessageDialog(null,"Problem Changing to Cone Model:"+e.getMessage(),"Couldn't change to Cone Model",JOptionPane.WARNING_MESSAGE);
                    selectComboBoxFromModule(s3m);
                }
                break;
            case 2:
                //Cube
                s3m.useCubeModel();
                break;
            case 3:
                //Teapot
                s3m.useTeapotModel();
                break;
            case 4:
                //Torus
                String itor = null;
                slices = null;
                stacks = null;
                
                itor = JOptionPane.showInputDialog("Inner to Outer Radius Ratio (<1)","0.5");
                if (itor != null)
                    slices = JOptionPane.showInputDialog("Slices","8");
                if (slices != null)
                    stacks = JOptionPane.showInputDialog("Stacks","16");
                
                try{
                    s3m.useTorusModel(Double.parseDouble(itor), Integer.parseInt(slices),Integer.parseInt(stacks));
                } catch (Exception e){
                    JOptionPane.showMessageDialog(null,"Problem Changing to Cone Model:"+e.getMessage(),"Couldn't change to Cone Model",JOptionPane.WARNING_MESSAGE);
                    selectComboBoxFromModule(s3m);
                }
                
                break;
            case 5:
                //Dodecahedron
                s3m.useDodecahedronModel();
                break;
            case 6:
                //Icosahedron
                s3m.useIcosahedronModel();
                break;
            case 7:
                //Octahedron
                s3m.useOctahedronModel();
                break;
            case 8:
                //Tetrahedron
                s3m.useTetrahedronModel();
                break;
            case 9:
                //TODO:implement
                throw new SDDMException("Custom module not implemented");
                //Custom
                
        }
    }
    
    private void selectScalingFromComboBox(Scalar3DModule moduleToSet) {
        switch(scalingComboBox.getSelectedIndex()) {
            case 1:
                //Logarithmic
                String lGainStr = JOptionPane.showInputDialog("Logarithmic scale factor? (a in ln(ax+b)+c","1.0");
                float lGain = Float.parseFloat(lGainStr);
                String lHorizOffStr = JOptionPane.showInputDialog("Logarithmic horizontal offset? (b in ln(ax+b)+c)","1.0");
                float lhOff = Float.parseFloat(lHorizOffStr);
                String lVertOffStr = JOptionPane.showInputDialog("Logarithmic vertical offset? (c in ln(ax+b)+c)","0.0");
                float lvOff = Float.parseFloat(lVertOffStr);
                moduleToSet.setLogarithmicScaling(lGain,lhOff,lvOff);
                break;
            case 2:
                //exponential
                String eGainStr = JOptionPane.showInputDialog("Exponential scale factor? (a in e^(ax+b)+c","1.0");
                float eGain = Float.parseFloat(eGainStr);
                String eHorizOffStr = JOptionPane.showInputDialog("Exponential horizontal offset? (b in e^(ax+b)+c)","0.0");
                float ehOff = Float.parseFloat(eHorizOffStr);
                String eVertOffStr = JOptionPane.showInputDialog("Exponential vertical offset? (c in e^(ax+b)+c)","-1.0");
                float evOff = Float.parseFloat(eVertOffStr);
                moduleToSet.setExponentialScaling(eGain,ehOff,evOff);
                break;
            case 3:
                //Polynomial
                String coeffsStr = JOptionPane.showInputDialog("Order of Polynomial?","2");
                float[] coeffs =  new float[Integer.parseInt(coeffsStr)];
                for(int i = 0; i < coeffs.length; ++i) {
                    String currStr = JOptionPane.showInputDialog("Value for "+i+"-order coefficeint?","1.0");
                    coeffs[i] = Float.parseFloat(currStr);
                }
                moduleToSet.setPolynomialScaling(coeffs);
                break;
            default:
                //linear for 0 and any accidental results
                moduleToSet.setLinearScaling();
                
        }
    }
    
    
    private void selectScalingFromModule(Scalar3DModule moduleToUse) {
        if (moduleToUse.isLogarithmicScaling())
            this.scalingComboBox.setSelectedIndex(1);
        else if (moduleToUse.isExponentialScaling())
            this.scalingComboBox.setSelectedIndex(2);
        else if (moduleToUse.isPolynomialScaling())
            this.scalingComboBox.setSelectedIndex(3);
        else
            this.scalingComboBox.setSelectedIndex(0);
    }
    
}
