/*
 * Vector3DModuleControlPanel.java
 *
 * Created on August 23, 2006, 1:15 PM
 */

package sddm.modules;

import sddm.*;
import net.java.joglutils.*;
import net.java.joglutils.lighting.*;
import javax.swing.*;
import java.awt.*;
import javax.vecmath.*;
/**
 *
 * @author  Erik J. Tollerud
 */
public class Vector3DModuleControlPanel extends SDDModuleControlPanel {
    Vector3DModule mod;
    private sddm.data.PointMap<Vector4f> resetToMap;
    
    /** Creates new form Vector3DModuleControlPanel */
    public Vector3DModuleControlPanel(Vector3DModule mod) {
        this.mod = mod;
        resetToMap = null;
        initComponents();
    }
    
    public SDDModule panelToModuleSettings(SDDModule module) {
        if (!(module instanceof Vector3DModule))
            throw new SDDMException("Tried to set module settings from a non-matched panel");
        
        Vector3DModule v3m = (Vector3DModule) module;
        v3m.setGranularity(((Number)this.granField.getValue()).intValue());
        v3m.setDrawPolygons(this.polyCheckBox.isSelected());
        v3m.setDrawOutline(this.outlineCheckBox.isSelected());
        v3m.setStipple(this.stippleCheckBox.isSelected());
        v3m.setLightingOn(this.lightingCheckBox.isSelected());
        v3m.setLightOutline(this.lightOlCheckBox.isSelected());
        v3m.getLight().setPhongShaded(this.phongCheckBox.isSelected());
        v3m.setShininess(((Number)this.shininessField.getValue()).floatValue());
        v3m.setGlobalScale(((Number)this.scaleField.getValue()).floatValue());
        v3m.setLineWidth(((Number)this.lineWidthField.getValue()).floatValue());
        v3m.setZeroBaseScaling(this.zeroScaleCheckBox.isSelected());
        v3m.setDrawStreamlines(this.slCheckBox.isSelected());
        v3m.setStreamlineWidth(((Number)this.slWidthField.getValue()).floatValue());
        selectModelFromComboBox(v3m);
        selectScalingFromComboBox(v3m);
        
        //selection setting
        if(selectCheckBox.isSelected()) {
            JFrame[] frames = this.getControlFrame().getAttachedFrames();
            for (JFrame frame: frames) {
                if (frame instanceof GLJFrame) {
                    v3m.setSelecting(((GLJFrame)frame).getAutoDrawable());
                    break;
                }
            }
        } else
            v3m.setSelecting(null);
        
        return v3m;
        
    }
    
    public void moduleSettingsToPanel(SDDModule module) {
        if (module != null) {
            if (!(module instanceof Vector3DModule))
                throw new SDDMException("Tried to set module settings from a non-matched panel");
            
            Vector3DModule v3m = (Vector3DModule) module;
            this.granField.setValue(v3m.getGranularity());
            this.polyCheckBox.setSelected(v3m.isDrawPolygons());
            this.outlineCheckBox.setSelected(v3m.isDrawOutline());
            this.stippleCheckBox.setSelected(v3m.isStipple());
            this.lightingCheckBox.setSelected(v3m.isLightingOn());
            this.lightOlCheckBox.setSelected(v3m.isLightOutline());
            this.phongCheckBox.setSelected(v3m.getLight().isPhongShaded());
            this.shininessField.setValue(v3m.getShininess());
            this.scaleField.setValue(v3m.getGlobalScale());
            this.lineWidthField.setValue(v3m.getLineWidth());
            this.zeroScaleCheckBox.setSelected(v3m.isZeroBaseScaling());
            this.slCheckBox.setSelected(v3m.isDrawStreamlines());
            this.slWidthField.setValue(v3m.getStreamlineWidth());
            
            if (v3m.isSelecting()) {
                this.selectCheckBox.setSelected(true);
                this.selectTextFieldActionPerformed(new java.awt.event.ActionEvent(this.selectTextField,0,""));
            } else
                this.selectCheckBox.setSelected(false);
            selectModelComboBoxFromModule(v3m);
            selectScalingFromModule(v3m);
            
        }
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {
        outlineColorButton = new javax.swing.JButton();
        outlineCheckBox = new javax.swing.JCheckBox();
        polyCheckBox = new javax.swing.JCheckBox();
        lightingButton = new javax.swing.JButton();
        jLabel2 = new javax.swing.JLabel();
        shininessField = new javax.swing.JFormattedTextField(java.text.NumberFormat.getInstance());
        lightingCheckBox = new javax.swing.JCheckBox();
        bgColorButton = new javax.swing.JButton();
        jLabel1 = new javax.swing.JLabel();
        granField = new javax.swing.JFormattedTextField(java.text.NumberFormat.getInstance());
        stippleCheckBox = new javax.swing.JCheckBox();
        polyColorButton = new javax.swing.JButton();
        phongCheckBox = new javax.swing.JCheckBox();
        scaleField = new javax.swing.JFormattedTextField(java.text.NumberFormat.getInstance());
        jLabel3 = new javax.swing.JLabel();
        lightOlCheckBox = new javax.swing.JCheckBox();
        perturbButton = new javax.swing.JButton();
        lineWidthField = new javax.swing.JFormattedTextField(java.text.NumberFormat.getInstance());
        jLabel4 = new javax.swing.JLabel();
        resetButton = new javax.swing.JButton();
        zeroScaleCheckBox = new javax.swing.JCheckBox();
        selectCheckBox = new javax.swing.JCheckBox();
        selectTextField = new javax.swing.JTextField();
        modelComboBox = new javax.swing.JComboBox();
        targCenterButton = new javax.swing.JButton();
        slButton = new javax.swing.JButton();
        slCheckBox = new javax.swing.JCheckBox();
        slColorButton = new javax.swing.JButton();
        saveSlButton = new javax.swing.JButton();
        loadSlButton = new javax.swing.JButton();
        slWidthField = new javax.swing.JFormattedTextField(java.text.NumberFormat.getInstance());
        jLabel5 = new javax.swing.JLabel();
        clearSlButton = new javax.swing.JButton();
        scalingComboBox = new javax.swing.JComboBox();
        jLabel6 = new javax.swing.JLabel();
        slGridButton = new javax.swing.JButton();

        setPreferredSize(new java.awt.Dimension(412, 330));
        outlineColorButton.setText("Outline Color Scheme...");
        outlineColorButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                outlineColorButtonActionPerformed(evt);
            }
        });

        outlineCheckBox.setSelected(mod.isDrawOutline());
        outlineCheckBox.setText("show Outline?");
        outlineCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        outlineCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        outlineCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                outlineCheckBoxActionPerformed(evt);
            }
        });

        polyCheckBox.setSelected(mod.isDrawPolygons());
        polyCheckBox.setText("show Polygon mesh?");
        polyCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        polyCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        polyCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                polyCheckBoxActionPerformed(evt);
            }
        });

        lightingButton.setText("Configure Light...");
        lightingButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                lightingButtonActionPerformed(evt);
            }
        });

        jLabel2.setText("shininess:");

        shininessField.setText("num");
        shininessField.setValue(mod.getShininess());
        shininessField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                shininessFieldActionPerformed(evt);
            }
        });

        lightingCheckBox.setSelected(this.mod.isLightingOn());
        lightingCheckBox.setText("enable Lighting");
        lightingCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        lightingCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        lightingCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                lightingCheckBoxActionPerformed(evt);
            }
        });

        bgColorButton.setText("Background Color...");
        bgColorButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                bgColorButtonActionPerformed(evt);
            }
        });

        jLabel1.setText("Granularity:");

        granField.setText("num");
        granField.setValue(mod.getGranularity());
        granField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                granFieldActionPerformed(evt);
            }
        });

        stippleCheckBox.setSelected(mod.isStipple());
        stippleCheckBox.setText("sTipple outline?");
        stippleCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        stippleCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        stippleCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                stippleCheckBoxActionPerformed(evt);
            }
        });

        polyColorButton.setText("Polygon Color Scheme...");
        polyColorButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                polyColorButtonActionPerformed(evt);
            }
        });

        phongCheckBox.setSelected(this.mod.getLight().isPhongShaded());
        phongCheckBox.setText("Phong lighting");
        phongCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        phongCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        phongCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                phongCheckBoxActionPerformed(evt);
            }
        });

        scaleField.setText("num");
        scaleField.setValue(mod.getGlobalScale());
        scaleField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                scaleFieldActionPerformed(evt);
            }
        });

        jLabel3.setText("scale:");

        lightOlCheckBox.setSelected(mod.isLightOutline());
        lightOlCheckBox.setText("light outline?");
        lightOlCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        lightOlCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        lightOlCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                lightOlCheckBoxActionPerformed(evt);
            }
        });

        perturbButton.setText("Random Perturbation...");
        perturbButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                perturbButtonActionPerformed(evt);
            }
        });

        lineWidthField.setText("num");
        lineWidthField.setValue(mod.getLineWidth());
        lineWidthField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                lineWidthFieldActionPerformed(evt);
            }
        });

        jLabel4.setText("Line width:");

        resetButton.setText("Reset");
        resetButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                resetButtonActionPerformed(evt);
            }
        });

        zeroScaleCheckBox.setSelected(mod.isZeroBaseScaling());
        zeroScaleCheckBox.setText("zero-base scaling");
        zeroScaleCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        zeroScaleCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        zeroScaleCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                zeroScaleCheckBoxActionPerformed(evt);
            }
        });

        selectCheckBox.setText("select?");
        selectCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        selectCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        selectCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                selectCheckBoxActionPerformed(evt);
            }
        });

        selectTextField.setEditable(false);
        selectTextField.setText("N/A");
        selectTextField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                selectTextFieldActionPerformed(evt);
            }
        });

        modelComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Block Arrow", "Wire  Arrow", "Cone", "Torus", "Custom" }));
        modelComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                modelComboBoxActionPerformed(evt);
            }
        });

        targCenterButton.setText("target center");
        targCenterButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                targCenterButtonActionPerformed(evt);
            }
        });

        slButton.setText("add Streamline...");
        slButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                slButtonActionPerformed(evt);
            }
        });

        slCheckBox.setText("show streamlines?");
        slCheckBox.setBorder(javax.swing.BorderFactory.createEmptyBorder(0, 0, 0, 0));
        slCheckBox.setMargin(new java.awt.Insets(0, 0, 0, 0));
        slCheckBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                slCheckBoxActionPerformed(evt);
            }
        });

        slColorButton.setText("Streamline Color...");
        slColorButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                slColorButtonActionPerformed(evt);
            }
        });

        saveSlButton.setText("Save...");
        saveSlButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                saveSlButtonActionPerformed(evt);
            }
        });

        loadSlButton.setText("Load...");
        loadSlButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                loadSlButtonActionPerformed(evt);
            }
        });

        slWidthField.setText("num");
        slWidthField.setValue(mod.getStreamlineWidth());
        slWidthField.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                slWidthFieldActionPerformed(evt);
            }
        });

        jLabel5.setText("Streamline width:");

        clearSlButton.setText("Clear Streamlines");
        clearSlButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                clearSlButtonActionPerformed(evt);
            }
        });

        scalingComboBox.setModel(new javax.swing.DefaultComboBoxModel(new String[] { "Linear", "Logarithmic", "Exponential", "Polynomial" }));
        scalingComboBox.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                scalingComboBoxActionPerformed(evt);
            }
        });

        jLabel6.setText("Scaling Method:");

        slGridButton.setText("Streamline Grid...");
        slGridButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                slGridButtonActionPerformed(evt);
            }
        });

        org.jdesktop.layout.GroupLayout layout = new org.jdesktop.layout.GroupLayout(this);
        this.setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                    .add(layout.createSequentialGroup()
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(layout.createSequentialGroup()
                                .add(polyColorButton)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(polyCheckBox))
                            .add(layout.createSequentialGroup()
                                .add(jLabel1)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(granField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(perturbButton)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(resetButton))
                            .add(layout.createSequentialGroup()
                                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(outlineColorButton)
                                    .add(bgColorButton))
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                                    .add(layout.createSequentialGroup()
                                        .add(stippleCheckBox)
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                        .add(lightOlCheckBox))
                                    .add(layout.createSequentialGroup()
                                        .add(outlineCheckBox)
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                        .add(jLabel4)
                                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                        .add(lineWidthField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))))
                            .add(layout.createSequentialGroup()
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(zeroScaleCheckBox)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jLabel6)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(scalingComboBox, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jLabel3)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(scaleField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                            .add(layout.createSequentialGroup()
                                .add(lightingButton)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(lightingCheckBox)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(phongCheckBox))
                            .add(layout.createSequentialGroup()
                                .add(selectCheckBox)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(selectTextField, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, 337, Short.MAX_VALUE)))
                        .addContainerGap())
                    .add(layout.createSequentialGroup()
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(jLabel2)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(shininessField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(modelComboBox, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                        .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                        .add(targCenterButton))
                    .add(layout.createSequentialGroup()
                        .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
                            .add(layout.createSequentialGroup()
                                .add(slButton)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(slCheckBox)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(jLabel5)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(slWidthField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                            .add(layout.createSequentialGroup()
                                .add(slGridButton)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(saveSlButton)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(loadSlButton)
                                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                                .add(slColorButton)))
                        .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
                    .add(layout.createSequentialGroup()
                        .add(clearSlButton)
                        .addContainerGap(289, Short.MAX_VALUE))))
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(org.jdesktop.layout.GroupLayout.LEADING)
            .add(layout.createSequentialGroup()
                .addContainerGap()
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(jLabel1)
                    .add(granField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(perturbButton)
                    .add(resetButton))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(polyColorButton)
                    .add(polyCheckBox))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(outlineColorButton)
                    .add(outlineCheckBox)
                    .add(jLabel4)
                    .add(lineWidthField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(bgColorButton)
                    .add(stippleCheckBox)
                    .add(lightOlCheckBox))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(lightingButton)
                    .add(lightingCheckBox)
                    .add(phongCheckBox))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(shininessField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel2)
                    .add(modelComboBox, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(targCenterButton))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(selectTextField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(selectCheckBox))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(zeroScaleCheckBox)
                    .add(jLabel6)
                    .add(scalingComboBox, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(scaleField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel3))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(slButton)
                    .add(slCheckBox)
                    .add(slWidthField, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE, org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, org.jdesktop.layout.GroupLayout.PREFERRED_SIZE)
                    .add(jLabel5))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(layout.createParallelGroup(org.jdesktop.layout.GroupLayout.BASELINE)
                    .add(slGridButton)
                    .add(saveSlButton)
                    .add(loadSlButton)
                    .add(slColorButton))
                .addPreferredGap(org.jdesktop.layout.LayoutStyle.RELATED)
                .add(clearSlButton)
                .addContainerGap(org.jdesktop.layout.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
        );
    }// </editor-fold>//GEN-END:initComponents

    private void slGridButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_slGridButtonActionPerformed
        
        float startX = Float.parseFloat(JOptionPane.showInputDialog("Start X?",mod.getMap().getXMin()));
        float startY = Float.parseFloat(JOptionPane.showInputDialog("Start Y?",mod.getMap().getYMin()));
        float startZ = Float.parseFloat(JOptionPane.showInputDialog("Start Z?",mod.getMap().getZMin()));
        float endX = Float.parseFloat(JOptionPane.showInputDialog("End X?",mod.getMap().getXMax()));
        float endY = Float.parseFloat(JOptionPane.showInputDialog("End Y?",mod.getMap().getYMax()));
        float endZ = Float.parseFloat(JOptionPane.showInputDialog("End Z?",mod.getMap().getZMax()));
        float spacing = Float.parseFloat(JOptionPane.showInputDialog("Spacing?",1.0f));
        
        java.util.ArrayList<float[]> initVals = new java.util.ArrayList<float[]>();
        
        for (float x = startX; x < endX; x+=spacing) {
            for (float y = startY; y < endY; y+=spacing) {
                for (float z = startZ; z < endZ; z+=spacing) {
                    float[] toAdd = new float[3];
                    toAdd[0] = x;
                    toAdd[1] = y;
                    toAdd[2] = z;
                    initVals.add(toAdd);
                }
            }
        }
        
        for (float[] val : initVals)
            mod.addStreamline(val);
        
        if(this.mod.isDrawStreamlines()) {
            this.redisplayParent();
        }
        
    }//GEN-LAST:event_slGridButtonActionPerformed
    
    private void scalingComboBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_scalingComboBoxActionPerformed
        try {
            selectScalingFromComboBox(this.mod);
            mod.recompileDisplayList();
            this.redisplayParent();
        } catch (Exception e) {
            JOptionPane.showMessageDialog(this,"Problem setting Scaling:"+e.getMessage(),"Scaling Error",JOptionPane.ERROR_MESSAGE);
            selectScalingFromModule(this.mod);
        }
    }//GEN-LAST:event_scalingComboBoxActionPerformed

    private void clearSlButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_clearSlButtonActionPerformed
        mod.clearStreamlines();
        mod.recompileDisplayList();
        this.redisplayParent();
    }//GEN-LAST:event_clearSlButtonActionPerformed
    
    private void slWidthFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_slWidthFieldActionPerformed
        mod.setStreamlineWidth(((Number)this.slWidthField.getValue()).floatValue());
        mod.recompileDisplayList();
        this.redisplayParent();
    }//GEN-LAST:event_slWidthFieldActionPerformed

    private void saveSlButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_saveSlButtonActionPerformed
        JFileChooser jfc = new JFileChooser();
        jfc.setMultiSelectionEnabled(false);
        jfc.setDialogTitle("Save Streamline");
        
        if(jfc.showSaveDialog(this)==JFileChooser.APPROVE_OPTION) {
            java.io.File f = jfc.getSelectedFile();
            
            try {
                if (!f.exists())
                    f.createNewFile();
                java.io.PrintWriter pw = new java.io.PrintWriter(f);
                for (Vector3DModule.Streamline sl : mod.getStreamlines()) {
                    float[] ics = sl.getICs();
                    if (ics.length == 3)
                        pw.println(ics[0]+" "+ics[1]+" "+ics[2]);
                    else if (ics.length == 4)
                        pw.println(ics[0]+" "+ics[1]+" "+ics[2]+" "+ics[3]);
                    else
                        JOptionPane.showMessageDialog(this,"Streamline does not have 3 or 4 initial conditions.","Malformed Streamline",JOptionPane.ERROR_MESSAGE);
                }
                pw.close();
            } catch(java.io.FileNotFoundException e){
                JOptionPane.showMessageDialog(this,"Could not find streamline file.","File Not Found",JOptionPane.ERROR_MESSAGE);
            } catch (java.io.IOException e) {
                JOptionPane.showMessageDialog(this,"Could not create file to save streamlines.","Couldn't Create File",JOptionPane.ERROR_MESSAGE);
            }
        }
    }//GEN-LAST:event_saveSlButtonActionPerformed
    
    private void loadSlButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loadSlButtonActionPerformed
        JFileChooser jfc = new JFileChooser();
        jfc.setMultiSelectionEnabled(false);
        jfc.setDialogTitle("Load Streamline");
        
        if(jfc.showOpenDialog(this)==JFileChooser.APPROVE_OPTION) {
            java.io.File f = jfc.getSelectedFile();
            
            try {
                float[] slAdder = null;
                java.util.Scanner sc = new java.util.Scanner(f);
                String firstLine = sc.nextLine();
                String[] elements = firstLine.split(" ");
                if (elements.length == 4) {
                    slAdder = new float[4];
                    for (int i = 0; i < 4;++i)
                        slAdder[i] = Float.parseFloat(elements[i]);
                }
                else if (elements.length == 3) {
                    slAdder = new float[3];
                    for (int i = 0; i < 3;++i)
                        slAdder[i] = Float.parseFloat(elements[i]);
                }
                else
                    throw new SDDMException("Streamline file lines not 3 or 4 elements.");
                mod.addStreamline(slAdder);
                
                int i = 1;
                while(sc.hasNextFloat()) {
                    slAdder[0] = sc.nextFloat();
                    slAdder[1] = sc.nextFloat();
                    slAdder[2] = sc.nextFloat();
                    if (slAdder.length == 4)
                        slAdder[3] = sc.nextFloat();
                    
                    mod.addStreamline(slAdder);
                }
                
            } catch(java.io.FileNotFoundException e){
                JOptionPane.showMessageDialog(this,"Could not find streamline file.","File Not Found",JOptionPane.ERROR_MESSAGE);
            } catch (Exception e) {
                JOptionPane.showMessageDialog(this,"Problem Loading streamlines:"+e.getMessage(),"Streamline Load Error",JOptionPane.ERROR_MESSAGE);
            }
        }
    }//GEN-LAST:event_loadSlButtonActionPerformed

    private void slColorButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_slColorButtonActionPerformed
        Coloring c = Coloring.coloringOptionPane(this);
        
        if (c!=null) {
            mod.setStreamlineColorer(c);
            mod.recompileDisplayList();
            this.redisplayParent();
        }
    }//GEN-LAST:event_slColorButtonActionPerformed

    private void slCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_slCheckBoxActionPerformed
        this.mod.setDrawStreamlines(slCheckBox.isSelected());
        this.mod.recompileDisplayList();
        this.redisplayParent();
    }//GEN-LAST:event_slCheckBoxActionPerformed

    private void slButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_slButtonActionPerformed
        String xVal = JOptionPane.showInputDialog("x-coordinate of initial point?",0.0f);
        String yVal = JOptionPane.showInputDialog("y-coordinate of initial point?",0.0f);
        String zVal = JOptionPane.showInputDialog("z-coordinate of initial point?",0.0f);
        
        float[] initial = {Float.parseFloat(xVal),Float.parseFloat(yVal),Float.parseFloat(zVal)};
        
        this.mod.addStreamline(initial);
        if(this.mod.isDrawStreamlines()) {
            this.mod.recompileDisplayList();
            this.redisplayParent();
        }
    }//GEN-LAST:event_slButtonActionPerformed

    private void targCenterButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_targCenterButtonActionPerformed
        sddm.data.PointMap<Vector4f> map = mod.getMap();
        cam.setTargetPoint(map.getXAvg(),map.getYAvg(),map.getZAvg());
        this.redisplayParent();
    }//GEN-LAST:event_targCenterButtonActionPerformed
    
    private void modelComboBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_modelComboBoxActionPerformed
        selectModelFromComboBox(this.mod);
        mod.recompileDisplayList();
        this.redisplayParent();
    }//GEN-LAST:event_modelComboBoxActionPerformed
    
    java.text.DecimalFormat idm = null;
    java.text.DecimalFormat vdm = null;
    private void selectTextFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_selectTextFieldActionPerformed
        
        
        if(mod.isSelecting()) {
            if (idm == null) {
                idm = new java.text.DecimalFormat();
                vdm = new java.text.DecimalFormat();
                idm.applyPattern("#####0.###");
                vdm.applyPattern("0.####E0");
            }
            
            selectTextField.setText(mod.getSelectionInfoString(idm,vdm," "));
            
            //TODO:more intelligent field sizing
            /*if (!selectTextField.getText().equals("Invalid Selection")){
                if (selectTextField.getPreferredSize().width > selectTextField.getWidth()) {
                    while(selectTextField.getPreferredSize().width > selectTextField.getWidth()) {
                        vdm.setMaximumFractionDigits(vdm.getMaximumFractionDigits()-1);
                        selectTextField.setText(mod.getSelectionInfoString(idm,vdm," "));
                    }
             
                }
                if (selectTextField.getPreferredSize().width < selectTextField.getWidth()) {
                    while(selectTextField.getPreferredSize().width < selectTextField.getWidth()) {
                        vdm.setMaximumFractionDigits(vdm.getMaximumFractionDigits()+1);
                        selectTextField.setText(mod.getSelectionInfoString(idm,vdm," "));
                    }
                }
                vdm.setMaximumFractionDigits(vdm.getMaximumFractionDigits()-1);
                selectTextField.setText(mod.getSelectionInfoString(idm,vdm," "));
            }*/
        } else
            selectTextField.setText("N/A");
    }//GEN-LAST:event_selectTextFieldActionPerformed
    
    private void selectCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_selectCheckBoxActionPerformed
        javax.media.opengl.GLAutoDrawable toApply = null;
        if(selectCheckBox.isSelected()) {
            JFrame[] frames = this.getControlFrame().getAttachedFrames();
            for (JFrame frame: frames) {
                if (frame instanceof GLJFrame) {
                    toApply = ((GLJFrame)frame).getAutoDrawable();
                    break;
                }
            }
        }
        //else leave null
        mod.setSelecting(toApply);
        selectCheckBox.setSelected(toApply != null);
        selectTextFieldActionPerformed(evt);
    }//GEN-LAST:event_selectCheckBoxActionPerformed
    
    private void zeroScaleCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_zeroScaleCheckBoxActionPerformed
        mod.setZeroBaseScaling(this.zeroScaleCheckBox.isSelected());
        this.redisplayParent();
    }//GEN-LAST:event_zeroScaleCheckBoxActionPerformed
    
    private void resetButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_resetButtonActionPerformed
        if (this.resetToMap != null) {
            mod.setMap(this.resetToMap);
            this.resetToMap = null;
            mod.recomputeStreamlines();
            mod.recompileDisplayList();
            this.redisplayParent();
        }
    }//GEN-LAST:event_resetButtonActionPerformed
    
    private void lineWidthFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_lineWidthFieldActionPerformed
        mod.setLineWidth(((Number)this.lineWidthField.getValue()).floatValue());
        mod.recompileDisplayList();
        this.redisplayParent();
    }//GEN-LAST:event_lineWidthFieldActionPerformed
    
    private void perturbButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_perturbButtonActionPerformed
        String result = JOptionPane.showInputDialog(null,"Maximum amount of Perturbation?","0.1");
        try {
            float scale = Float.parseFloat(result);
            if (resetToMap == null)
                resetToMap = mod.getMap().clone();
            mod.getMap().randomPerturbation(scale,false);
            mod.recomputeStreamlines();
            mod.recompileDisplayList();
            this.redisplayParent();
        } catch (Exception e) {
            JOptionPane.showMessageDialog(null,"Problem performing perturbation:"+e.getMessage(),"Did not perturb",JOptionPane.ERROR_MESSAGE);
            //just do nothing if there's a problem here
        }
    }//GEN-LAST:event_perturbButtonActionPerformed
    
    private void lightOlCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_lightOlCheckBoxActionPerformed
        mod.setLightOutline(lightOlCheckBox.isSelected());
        mod.recompileDisplayList();
        this.redisplayParent();
    }//GEN-LAST:event_lightOlCheckBoxActionPerformed
    
    private void scaleFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_scaleFieldActionPerformed
        float scale = ((Number)this.scaleField.getValue()).floatValue();
        if (scale < 0 ) {
            mod.setGlobalScaleToFillBox(-scale);
            selectScalingFromModule(mod);
            this.scaleField.setText(Float.toString(mod.getGlobalScale()));
        } else
            mod.setGlobalScale(scale);
        
        mod.recompileDisplayList();
        this.redisplayParent();
    }//GEN-LAST:event_scaleFieldActionPerformed
    
    private void outlineColorButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_outlineColorButtonActionPerformed
        Coloring c = Coloring.coloringOptionPane(this);
        
        if (c!=null) {
            mod.setOutlineColoring(c);
            mod.recompileDisplayList();
            this.redisplayParent();
        }
    }//GEN-LAST:event_outlineColorButtonActionPerformed
    
    private void polyColorButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_polyColorButtonActionPerformed
        Coloring c = Coloring.coloringOptionPane(this);
        
        if (c!=null) {
            mod.setPolygonColoring(c);
            mod.recompileDisplayList();
            this.redisplayParent();
        }
    }//GEN-LAST:event_polyColorButtonActionPerformed
    
    private void phongCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_phongCheckBoxActionPerformed
        try {
            mod.getLight().setPhongShaded(this.phongCheckBox.isSelected());
            
            mod.recompileDisplayList();
            this.redisplayParent();
        } catch(LightingException e) {
            JOptionPane.showMessageDialog(null,"Problem setting Phong shader: "+e.getMessage(),"Phong Shading Failed",JOptionPane.WARNING_MESSAGE);
        }
    }//GEN-LAST:event_phongCheckBoxActionPerformed
    
    private void stippleCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stippleCheckBoxActionPerformed
        mod.setStipple(stippleCheckBox.isSelected());
        this.redisplayParent();
    }//GEN-LAST:event_stippleCheckBoxActionPerformed
    
    private void granFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_granFieldActionPerformed
        mod.setGranularity(((Number)granField.getValue()).intValue());
        mod.recompileDisplayList();
        this.redisplayParent();
    }//GEN-LAST:event_granFieldActionPerformed
    
    private void bgColorButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_bgColorButtonActionPerformed
        Color selectedColor = new JColorChooser().showDialog(this,"Background Color?",mod.getBackgroundColor());
        if (selectedColor != null) {
            mod.setBackgroundColor(selectedColor);
            this.redisplayParent();
        }
    }//GEN-LAST:event_bgColorButtonActionPerformed
    
    private void lightingCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_lightingCheckBoxActionPerformed
        this.mod.setLightingOn(lightingCheckBox.isSelected());
        this.redisplayParent();
    }//GEN-LAST:event_lightingCheckBoxActionPerformed
    
    private void shininessFieldActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_shininessFieldActionPerformed
        try {
            mod.setShininess(((Number)shininessField.getValue()).floatValue());
            this.redisplayParent();
        } catch (SDDMException e) {
            JOptionPane.showMessageDialog(null,"Tried to set shininess greater than 128 or less than 0.  OpenGL Shininesss must be between 0 and 128, inclusive.","Couldn't set shininess",JOptionPane.ERROR_MESSAGE);
            shininessField.setValue(mod.getShininess());
        }
    }//GEN-LAST:event_shininessFieldActionPerformed
    
    private void lightingButtonActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_lightingButtonActionPerformed
        LightPanel lp = new LightPanel(mod.getLight());
        JPanelDialog jpd = new JPanelDialog(lp);
        ColorButton cb = new ColorButton("Global Ambient");
        cb.setColor(mod.getGlobalAmbient());
        jpd.add(cb);
        if(jpd.showAsModal()) {
            lp.panelToLight(mod.getLight());
            mod.setGlobalAmbient(cb.getColor());
            if (mod.isLightingOn()) {
                mod.recompileDisplayList();
                this.redisplayParent();
            }
        }
    }//GEN-LAST:event_lightingButtonActionPerformed
    
    private void polyCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_polyCheckBoxActionPerformed
        mod.setDrawPolygons(polyCheckBox.isSelected());
        mod.recompileDisplayList();
        this.redisplayParent();
    }//GEN-LAST:event_polyCheckBoxActionPerformed
    
    private void outlineCheckBoxActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_outlineCheckBoxActionPerformed
        mod.setDrawOutline(outlineCheckBox.isSelected());
        mod.recompileDisplayList();
        this.redisplayParent();
    }//GEN-LAST:event_outlineCheckBoxActionPerformed
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton bgColorButton;
    private javax.swing.JButton clearSlButton;
    private javax.swing.JFormattedTextField granField;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JLabel jLabel5;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JCheckBox lightOlCheckBox;
    private javax.swing.JButton lightingButton;
    private javax.swing.JCheckBox lightingCheckBox;
    private javax.swing.JFormattedTextField lineWidthField;
    private javax.swing.JButton loadSlButton;
    private javax.swing.JComboBox modelComboBox;
    private javax.swing.JCheckBox outlineCheckBox;
    private javax.swing.JButton outlineColorButton;
    private javax.swing.JButton perturbButton;
    private javax.swing.JCheckBox phongCheckBox;
    private javax.swing.JCheckBox polyCheckBox;
    private javax.swing.JButton polyColorButton;
    private javax.swing.JButton resetButton;
    private javax.swing.JButton saveSlButton;
    private javax.swing.JFormattedTextField scaleField;
    private javax.swing.JComboBox scalingComboBox;
    private javax.swing.JCheckBox selectCheckBox;
    private javax.swing.JTextField selectTextField;
    private javax.swing.JFormattedTextField shininessField;
    private javax.swing.JButton slButton;
    private javax.swing.JCheckBox slCheckBox;
    private javax.swing.JButton slColorButton;
    private javax.swing.JButton slGridButton;
    private javax.swing.JFormattedTextField slWidthField;
    private javax.swing.JCheckBox stippleCheckBox;
    private javax.swing.JButton targCenterButton;
    private javax.swing.JCheckBox zeroScaleCheckBox;
    // End of variables declaration//GEN-END:variables
    public void paint(java.awt.Graphics g) {
        this.moduleSettingsToPanel(mod);
        super.paint(g);
    }
    
//-----------------------------------------privates below here------------------------------
    private void selectModelComboBoxFromModule(Vector3DModule v3m){
        java.awt.event.ActionListener[] als = this.modelComboBox.getActionListeners();
        for (java.awt.event.ActionListener l : als)
            this.modelComboBox.removeActionListener(l);
        
        switch(v3m.getModelType()) {
            case BLOCK_ARROW:
                this.modelComboBox.setSelectedIndex(0);
                break;
            case WIRE_ARROW:
                this.modelComboBox.setSelectedIndex(1);
                break;
            case CONE:
                this.modelComboBox.setSelectedIndex(2);
                break;
            case TORUS:
                this.modelComboBox.setSelectedIndex(3);
                break;
            case CUSTOM:
                this.modelComboBox.setSelectedIndex(4);
        }
        
        for (java.awt.event.ActionListener l : als)
            this.modelComboBox.addActionListener(l);
    }
    
    private void selectModelFromComboBox(Vector3DModule v3m) {
        switch(this.modelComboBox.getSelectedIndex()) {
            case 0:
                //Block Arrow
                v3m.useBlockArrowModel();
                break;
            case 1:
                //Wire Arrow
                v3m.useWireArrowModel();
                break;
            case 2:
                //Cone
                String bthr = null;
                String slices = null;
                String stacks = null;
                
                bthr = JOptionPane.showInputDialog("Base/Height Ratio","0.5");
                if (bthr != null)
                    slices = JOptionPane.showInputDialog("Slices","8");
                if (slices != null)
                    stacks = JOptionPane.showInputDialog("Stacks","8");
                
                try{
                    v3m.useConeModel(Double.parseDouble(bthr), Integer.parseInt(slices),Integer.parseInt(stacks));
                } catch (Exception e){
                    JOptionPane.showMessageDialog(null,"Problem Changing to Cone Model:"+e.getMessage(),"Couldn't change to Cone Model",JOptionPane.WARNING_MESSAGE);
                    selectModelComboBoxFromModule(v3m);
                }
                break;
            case 3:
                //Torus
                String itor = null;
                slices = null;
                stacks = null;
                
                itor = JOptionPane.showInputDialog("Inner to Outer Radius Ratio (<1)","0.5");
                if (itor != null)
                    slices = JOptionPane.showInputDialog("Slices","8");
                if (slices != null)
                    stacks = JOptionPane.showInputDialog("Stacks","16");
                
                try{
                    v3m.useTorusModel(Double.parseDouble(itor), Integer.parseInt(slices),Integer.parseInt(stacks));
                } catch (Exception e){
                    JOptionPane.showMessageDialog(null,"Problem Changing to Cone Model:"+e.getMessage(),"Couldn't change to Cone Model",JOptionPane.WARNING_MESSAGE);
                    selectModelComboBoxFromModule(v3m);
                }
                
                break;
            case 4:
                //TODO:implement
                throw new SDDMException("Custom drawing from setup panel not implemented");
                //Custom
        }
    }
    
    private void selectScalingFromComboBox(Vector3DModule moduleToSet) {
        switch(scalingComboBox.getSelectedIndex()) {
            case 1:
                //Logarithmic
                String lGainStr = JOptionPane.showInputDialog("Logarithmic scale factor? (a in ln(ax+b)+c","1.0");
                float lGain = Float.parseFloat(lGainStr);
                String lHorizOffStr = JOptionPane.showInputDialog("Logarithmic horizontal offset? (b in ln(ax+b)+c)","1.0");
                float lhOff = Float.parseFloat(lHorizOffStr);
                String lVertOffStr = JOptionPane.showInputDialog("Logarithmic vertical offset? (c in ln(ax+b)+c)","0.0");
                float lvOff = Float.parseFloat(lVertOffStr);
                moduleToSet.setLogarithmicScaling(lGain,lhOff,lvOff);
                break;
            case 2:
                //exponential
                String eGainStr = JOptionPane.showInputDialog("Exponential scale factor? (a in e^(ax+b)+c","1.0");
                float eGain = Float.parseFloat(eGainStr);
                String eHorizOffStr = JOptionPane.showInputDialog("Exponential horizontal offset? (b in e^(ax+b)+c)","0.0");
                float ehOff = Float.parseFloat(eHorizOffStr);
                String eVertOffStr = JOptionPane.showInputDialog("Exponential vertical offset? (c in e^(ax+b)+c)","-1.0");
                float evOff = Float.parseFloat(eVertOffStr);
                moduleToSet.setExponentialScaling(eGain,ehOff,evOff);
                break;
            case 3:
                //Polynomial
                String coeffsStr = JOptionPane.showInputDialog("Order of Polynomial?","2");
                float[] coeffs =  new float[Integer.parseInt(coeffsStr)+1];
                for(int i = 0; i < coeffs.length; ++i) {
                    String currStr = JOptionPane.showInputDialog("Value for "+i+"-order coefficeint?","1.0");
                    coeffs[i] = Float.parseFloat(currStr);
                }
                moduleToSet.setPolynomialScaling(coeffs);
                break;
            default:
                //linear for 0 and any accidental results
                moduleToSet.setLinearScaling();
                
        }
    }
    
    
    private void selectScalingFromModule(Vector3DModule moduleToUse) {
        java.awt.event.ActionListener al = this.scalingComboBox.getActionListeners()[0];
        this.scalingComboBox.removeActionListener(al);
        if (moduleToUse.isLogarithmicScaling())
            this.scalingComboBox.setSelectedIndex(1);
        else if (moduleToUse.isExponentialScaling())
            this.scalingComboBox.setSelectedIndex(2);
        else if (moduleToUse.isPolynomialScaling())
            this.scalingComboBox.setSelectedIndex(3);
        else
            this.scalingComboBox.setSelectedIndex(0);
        this.scalingComboBox.addActionListener(al);
    }

}
